@@ -3,7 +3,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-#include <time.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
@@ -28,19 +27,20 @@
 
 
 /* Utilities */
-static gchar *FORMATED_NUMBER_STRING(gulong i);
-static gchar *TIME_LAPSED_STRING(gulong dt);
-static void PRINT_ERR(const gchar *path, const gchar *mesg);
+static void PRINT_ERR(const gchar *msg);
 static void PRINT_SCANNING_FILE(
-	const gulong i, const gulong n, const gchar *path
+	const gchar *path,
+	const gulong i, const gulong n
+);
+static void PRINT_SCANNED(
+	const gulong files_scanned,
+	const gulong data_scanned_blocks,
+	const gulong total_data_scanned_blocks
 );
 static void PRINT_SCANNING_DIR(const gchar *path);
-static void PRINT_REPORT(
-	gulong time_start, gulong time_end,
-	gulong infected_files,
-	gulong size_scanned_mb,
-	gulong files_scanned 
-);
+static void PRINT_CLEAN(const gchar *path);
+static void PRINT_INFECTED(const gchar *path, const gchar *virus);
+static void PRINT_PROBLEM(const gchar *path, const gchar *problem);
 
 static gchar *AVScanCopyShortenString(const gchar *s, const gint m);
 
@@ -54,22 +54,27 @@ typedef struct {
 	struct cl_stat	database_stat;		/* Database stats, for
 						 * checking database changes */
 	gulong		files_scanned,		/* Files scanned so far */
-			size_scanned_mb,
+			size_scanned_blocks,
 			infected_files,
 			total_files;		/* Total files to be scanned */
 } ClamAVData;
-static ClamAVData *AVScanOPClamAVInit(const gchar *db_path);
+static ClamAVData *AVScanOPClamAVInit(
+	const gchar *db_path,
+	gint *status
+);
 static void AVScanOPClamAVShutdown(ClamAVData *d);
 static gint AVScanOPClamAVScanFile(
 	ClamAVData *d, const gchar *path,
-	gint *stop_count
+	gint *stop_count,
+	scanner_opts_struct *scanner_opts
 );
 static gint AVScanOPClamAVScanDir(
 	ClamAVData *d, const gchar *path,
 	const gboolean recursive,
 	const gboolean executables_only,
 	const gboolean ignore_links,
-	gint *stop_count
+	gboolean *reload, gint *stop_count,
+	scanner_opts_struct *scanner_opts
 );
 #endif	/* HAVE_CLAMAV */
 
@@ -96,7 +101,8 @@ gint AVScanOP(
 	const gboolean recursive,
 	const gboolean executables_only,
 	const gboolean ignore_links,
-	gint *stop_count
+	gboolean *reload, gint *stop_count,
+	scanner_opts_struct *scanner_opts
 );
 
 
@@ -112,154 +118,17 @@ gint AVScanOP(
 #define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
-/*
- *	Returns a dynamically allocated string describing the
- *	specified number.
- */
-static gchar *FORMATED_NUMBER_STRING(gulong i)
-{
-	gint comma_cnt, slen;
-	gchar *ss, *ss_ptr, *ts, *ts_ptr;
-
-	ss = g_strdup_printf("%ld", i);
-	slen = STRLEN(ss);
-			  
-	/* 3 digits or less? (no commas needed) */
-	if((slen <= 3) || (ss == NULL))
-	    return(ss);
-
-	ts_ptr = ts = (gchar *)g_malloc(80);
-	if(ts == NULL)
-	    return(ss);
-
-	ss_ptr = ss;
-
-	/* Initialize comma counter */
-	comma_cnt = slen % 3;
-	if(comma_cnt <= 0)
-	    comma_cnt = 3;
-
-	/* Iterate through size string until end is reached */
-	while(*ss_ptr != '\0')
-	{
-	    /* Reset comma counter and put in a comma? */
-	    if(comma_cnt <= 0)
-	    {
-		*ts_ptr = ',';
-		ts_ptr++;
-		comma_cnt = 3;
-	    }
-
-	    *ts_ptr = *ss_ptr;
-	    ts_ptr++;
-	    ss_ptr++;
-	    comma_cnt--;
-	}
-
-	/* Null terminate return string */
-	*ts_ptr = '\0';
-
-	g_free(ss);
-
-	return(ts);
-}
-
-/*
- *	Returns a dynamically allocated string describing the time
- *	lapsed specified by dt.
- */
-static gchar *TIME_LAPSED_STRING(gulong dt)
-{
-	gchar *s;
-
-	/* Less than one second? */   
-	if(dt < 1)
-	{
-	    s = g_strdup("less than one second");
-	}
-	/* Less than one minute? */
-	else if(dt < (1 * 60))
-	{
-	    const gulong ct = MAX(dt / 1, 1);
-	    s = g_strdup_printf(
-		"%ld %s",
-		ct, ((ct == 1l) ? "second" : "seconds")
-	    );
-	}
-	/* Less than one hour? */
-	else if(dt < (60 * 60))
-	{
-	    const gulong	ct = MAX(dt / 60, 1),
-				ct2 = MAX(dt / 1, 1) % 60;
-	    s = g_strdup_printf(
-		"%ld %s %ld %s",
-		ct, ((ct == 1l) ? "minute" : "minutes"),
-		ct2, ((ct2 == 1l) ? "second" : "seconds")
-	    );
-	}
-	/* Less than one day? */
-	else if(dt < (24 * 60 * 60))
-	{
-	    const gulong	ct = MAX(dt / 60 / 60, 1),
-				ct2 = MAX(dt / 60, 1) % 60;
-	    s = g_strdup_printf(
-		"%ld %s %ld %s",
-		ct, ((ct == 1l) ? "hour" : "hours"),
-		ct2, ((ct2 == 1l) ? "minute" : "minutes")
-	    );
-	}
-	/* Less than one week? */     
-	else if(dt < (7 * 24 * 60 * 60))
-	{
-	    const gulong	ct = MAX(dt / 60 / 60 / 24, 1),
-				ct2 = MAX(dt / 60 / 60, 1) % 24;
-	    s = g_strdup_printf(
-		"%ld %s %ld %s",
-		ct, ((ct == 1l) ? "day" : "days"), 
-		ct2, ((ct2 == 1l) ? "hour" : "hours")
-	    );
-	}
-	/* Less than one month (30 days)? */
-	else if(dt < (30 * 24 * 60 * 60))
-	{
-	    const gulong	ct = MAX(dt / 60 / 60 / 24 / 7, 1),
-				ct2 = MAX(dt / 60 / 60 / 24, 1) % 7;
-	    s = g_strdup_printf(
-		"%ld %s %ld %s",
-		ct, ((ct == 1l) ? "week" : "weeks"),
-		ct2, ((ct2 == 1l) ? "day" : "days")
-	    );
-	}
-	/* Less than 6 months ago? */
-#if 0
-	else if(dt < (6 * 30 * 24 * 60 * 60))
-#else
-	else
-#endif
-	{
-	    const gulong	ct = MAX(dt / 60 / 60 / 24 / 30, 1),
-				ct2 = MAX(dt / 60 / 60 / 24, 1) % 30;
-	    s = g_strdup_printf(
-		"%ld %s %ld %s",
-		ct, ((ct == 1l) ? "month" : "months"),
-		ct2, ((ct2 == 1l) ? "day" : "days")
-	    );
-	}
-
-	return(s);
-}
-
 
 /*
  *	Prints the specified path and message to stderr.
  *
  *	This is intended to indicate and have the monitoring process
  *	record the object specified by path as having a virus or
- *	other problem specified by mesg.
+ *	other problem specified by msg.
  */
-static void PRINT_ERR(const gchar *path, const gchar *mesg)
+static void PRINT_ERR(const gchar *msg)
 {
-	g_printerr("\"%s\" %s\n", path, mesg);
+	g_printerr("%s\n", msg);
 }
 
 /*
@@ -267,97 +136,49 @@ static void PRINT_ERR(const gchar *path, const gchar *mesg)
  *	it is being scanned.
  *
  *	This is intended to indicate and have the monitoring process
- *	display this message on its status.
+ *	display this message on its status bar.
  */
 static void PRINT_SCANNING_FILE(
-	const gulong i, const gulong n, const gchar *path
+	const gchar *path,
+	const gulong i, const gulong n
 )
 {
 	g_print("Scanning file: \"%s\" %ld %ld\n", path, i, n);
 }
+
+static void PRINT_SCANNED(
+	const gulong files_scanned,
+	const gulong data_scanned_blocks,
+	const gulong total_data_scanned_blocks
+)
+{
+	g_print("Scanned file: %ld %ld %ld\n",
+	    files_scanned,
+	    data_scanned_blocks, total_data_scanned_blocks
+	);
+}
+
 static void PRINT_SCANNING_DIR(const gchar *path)
 {
 	g_print("Scanning directory: \"%s\"\n", path);
 }
 
-/*
- *	Prints the final report in a message to stdout.
- *
- *	This is intended to indicate and have the monitoring process
- *	display this message in a dialog.
- */
-static void PRINT_REPORT(
-	gulong time_start, gulong time_end,
-	gulong infected_files,
-	gulong size_scanned_mb,
-	gulong files_scanned
-)
+static void PRINT_CLEAN(const gchar *path)
 {
-	time_t t = (time_t)time_start;
-	gchar	*s,
-		*time_lapsed_s = TIME_LAPSED_STRING(
-			time_end - time_start
-		),
-		*time_start_s = STRDUP(ctime(&t)),
-		*infected_files_s = FORMATED_NUMBER_STRING(
-		    infected_files
-		),
-		*size_scanned_mb_s = FORMATED_NUMBER_STRING(
-		    size_scanned_mb
-		),
-		*files_scanned_s = FORMATED_NUMBER_STRING(
-		    files_scanned
-		);
-
-	/* Remove tailing newline character from time_start_s */
-	s = strchr(time_start_s, '\n');
-	if(s != NULL)
-	    *s = '\0';
-
-	/* Note that all ';' characters indicate newlines for the
-	 * monitoring process that reads this
-	 */
-	if(infected_files > 0)
-	    g_print(
-"ReportInfected:\
-Started On: %s;\
-Duration: %s;\
-;\
-Infected Files: %s;\
-Files Scanned: %s;\
-Data Scanned: %s mb;\
-;\
-DO NOT VIEW OR RUN THE INFECTED FILES;\
-\n",
-		time_start_s,
-		time_lapsed_s,
-		infected_files_s,
-		files_scanned_s,
-		size_scanned_mb_s
-	    );
-	else
-	    g_print(
-"Report:\
-Started On: %s;\
-Duration: %s;\
-;\
-No infected files found;\
-Files Scanned: %s;\
-Data Scanned: %s mb;\
-\n",
-		time_start_s,
-		time_lapsed_s,
-		files_scanned_s,
-		size_scanned_mb_s
-	    );
-
-	g_free(time_lapsed_s);
-	g_free(time_start_s);
-	g_free(infected_files_s);
-	g_free(size_scanned_mb_s);
-	g_free(files_scanned_s);
+	g_print("Clean: \"%s\"\n", path);
 }
 
+static void PRINT_INFECTED(const gchar *path, const gchar *virus)
+{
+	g_print("Infected: \"%s\" %s\n", path, virus);
+}
+
+static void PRINT_PROBLEM(const gchar *path, const gchar *problem)
+{
+	g_print("Problem: \"%s\" %s\n", path, problem);
+}
+
+
 /*
  *	Returns a copy of the string that is no longer than max
  *	characters with appropriate shortened notation where 
@@ -391,9 +212,12 @@ static gchar *AVScanCopyShortenString(const gchar *s, const gint m)
 /*
  *	Initializes the Clam AntiVirus.
  */
-static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
+static ClamAVData *AVScanOPClamAVInit(
+	const gchar *db_path,
+	gint *status
+)
 {
-	gint status;
+	gint status2;
 	gchar *s;
 	struct stat stat_buf;
 	ClamAVData *d = (ClamAVData *)g_malloc0(
@@ -401,12 +225,13 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	);
 	if(d == NULL)
 	{
-	    g_print("Memory allocation error\n");
+	    PRINT_ERR("Memory allocation error");
+	    *status = 3;
 	    return(NULL);
 	}
 
 	d->files_scanned = 0l;
-	d->size_scanned_mb = 0l;
+	d->size_scanned_blocks = 0l;
 	d->infected_files = 0l;
 	d->total_files = 0l;
 
@@ -417,26 +242,47 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	    db_path = cl_retdbdir();
 	if(db_path == NULL)
 	{
-	    g_print("Virus database location was not specified\n");
+	    PRINT_ERR("Virus database location was not specified");
 	    g_free(d);
+	    *status = 50;
 	    return(NULL);
 	}
 	d->database_path = STRDUP(db_path);
 	if(d->database_path == NULL)
 	{
-	    g_print("Memory allocation error\n");
+	    PRINT_ERR("Memory allocation error");
 	    g_free(d);
+	    *status = 3;
 	    return(NULL);
 	}
 
-	/* Stat and load the virus database */ 
+	/* Get the virus database's stats */
+	if(stat((const char *)d->database_path, &stat_buf))
+	{
+	    const gint error_code = (gint)errno;
+	    gchar *error_msg = STRDUP(g_strerror(error_code));
+	    if(error_msg != NULL)
+	    {
+		*error_msg = (gchar)toupper((int)*error_msg);
+		PRINT_PROBLEM(d->database_path, error_msg);
+		g_free(error_msg);
+	    }
+	    g_free(d->database_path);
+	    g_free(d);
+	    *status = 50;
+	    return(NULL);
+	}
+
+	/* Load the virus database */ 
 	s = AVScanCopyShortenString(d->database_path, 50);
 	g_print("Loading virus database \"%s\"...\n", s);
 	g_free(s);
+
+	d->database = NULL;
+	d->database_entries = 0;
+
 #ifdef S_ISDIR
-	if((!stat(d->database_path, &stat_buf)) ?
-	    S_ISDIR(stat_buf.st_mode) : FALSE
-	)
+	if(S_ISDIR(stat_buf.st_mode))
 #else
 	if(FALSE)
 #endif
@@ -444,17 +290,16 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	    d->database_dir = STRDUP(d->database_path);
 	    if(d->database_dir == NULL)
 	    {
-		g_print("Memory allocation error\n");
+		PRINT_ERR("Memory allocation error");
 		g_free(d->database_path);
 		g_free(d);
+		*status = 3;
 		return(NULL);
 	    }
 
-	    memset(&d->database_stat, 0x00, sizeof(struct cl_stat));
-	    cl_statinidir(d->database_dir, &d->database_stat);
-
-	    status = cl_loaddbdir(
-		d->database_path, &d->database, &d->database_entries
+	    status2 = cl_loaddbdir(
+		d->database_path,
+		&d->database, &d->database_entries
 	    );
 	}
 	else
@@ -462,41 +307,70 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	    d->database_dir = g_dirname(d->database_path);
 	    if(d->database_dir == NULL)
 	    {
-		g_print("Memory allocation error\n");
+		PRINT_ERR("Memory allocation error");
 		g_free(d->database_path);
 		g_free(d);
+		*status = 3;
 		return(NULL);
 	    }
 
-	    memset(&d->database_stat, 0x00, sizeof(struct cl_stat));
-	    cl_statinidir(d->database_dir, &d->database_stat);
-
-	    status = cl_loaddb(
-		d->database_path, &d->database, &d->database_entries
+	    status2 = cl_loaddb(
+		d->database_path,
+		&d->database, &d->database_entries
 	    );
 	}
-	if(status || (d->database == NULL) || (d->database_entries <= 0))
+	if((status2 != 0) || (d->database == NULL) || (d->database_entries <= 0))
 	{
-	    g_print(
-"Failed to load the virus database\n"
+	    gchar *s = g_strconcat(
+		"Virus database load error: ",
+		cl_strerror(status2),
+		NULL
 	    );
-	    cl_statfree(&d->database_stat);
+	    PRINT_PROBLEM(d->database_path, s);
+	    g_free(s);
+	    if(d->database != NULL)
+		cl_free(d->database);
 	    g_free(d->database_path);
 	    g_free(d->database_dir);
 	    g_free(d);
+	    *status = 50;
 	    return(NULL);
 	}
 
 	/* Build the virus database */
-	if(cl_build(d->database))
+	status2 = cl_build(d->database);
+	if(status2 != 0)
 	{
-	    g_print(
-"Failed to build the virus database\n"
+	    gchar *s = g_strconcat(
+		"Virus database build error: ",
+		cl_strerror(status2),
+		NULL
 	    );
-	    cl_statfree(&d->database_stat);
+	    PRINT_PROBLEM(d->database_path, s);
+	    g_free(s);
+	    if(d->database != NULL)
+		cl_free(d->database);
 	    g_free(d->database_path);
 	    g_free(d->database_dir);
 	    g_free(d);
+	    *status = 50;
+	    return(NULL);
+	}
+
+	/* Get the virus database directory stats */
+	memset(&d->database_stat, 0x00, sizeof(d->database_stat));
+	if(cl_statinidir(d->database_dir, &d->database_stat))
+	{
+	    PRINT_PROBLEM(
+		d->database_dir,
+		"Unable to get the virus database directory statistics"
+	    );
+	    if(d->database != NULL)
+		cl_free(d->database);
+	    g_free(d->database_path);
+	    g_free(d->database_dir);
+	    g_free(d);
+	    *status = 50;
 	    return(NULL);
 	}
 
@@ -529,48 +403,39 @@ static void AVScanOPClamAVShutdown(ClamAVData *d)
  */
 static gint AVScanOPClamAVScanFile(
 	ClamAVData *d, const gchar *path,
-	gint *stop_count
+	gint *stop_count,
+	scanner_opts_struct *scanner_opts
 )
 {
+	unsigned long int scanned_blocks = 0l;
+	struct cl_limits limits;
 	gint status, fd;
 	const gchar *virus_name = NULL;
 	guint options;
-	gulong size_mb = 0l;
-	struct cl_limits limits;
 
 	/* Interrupted? */
 	if(*stop_count > 0)
 	    return(4);
 
-	PRINT_SCANNING_FILE(d->files_scanned, d->total_files, path);
+	PRINT_SCANNING_FILE(
+	    path,
+	    d->files_scanned,
+	    d->total_files
+	);
 
 	/* Open the file for reading */
-	fd = open(path, O_RDONLY);
+	fd = open((const char *)path, O_RDONLY);
 	if(fd < 0)
 	{
-	    const gint error_code = errno;
+	    const gint error_code = (gint)errno;
 	    gchar *error_msg = STRDUP(g_strerror(error_code));
 	    if(error_msg != NULL)
 	    {
-		gchar *s;
-
 		*error_msg = (gchar)toupper((int)*error_msg);
-		s = g_strdup_printf(
-		    "%s",
-		    error_msg
-		);
-		PRINT_ERR(path, s);
-		g_free(s);
+		PRINT_PROBLEM(path, error_msg);
 		g_free(error_msg);
 	    }
-	    else
-	    {
-		PRINT_ERR(
-		    path,
-		    "Unable to open the file for reading"
-		);
-	    }
-	    return(1);
+	    return(54);
 	}
 
 	/* Interrupted? */
@@ -583,9 +448,25 @@ static gint AVScanOPClamAVScanFile(
 
 	/* Set up archive limits */
 	memset(&limits, 0x00, sizeof(struct cl_limits));
-	limits.maxfiles = 50000;	/* Maximum files to scan within an archive */
-	limits.maxfilesize = 10 * 1048576;	/* Maximal archived file size == 10 Mb */
-	limits.maxreclevel = 8;		/* Maximal recursion level */
+
+	/* Maximum files to scan within an archive */
+	limits.maxfiles = (unsigned int)scanner_opts->max_files;
+
+	/* Maximum archived file size in bytes */
+	limits.maxfilesize = (unsigned long int)scanner_opts->max_file_size_bytes;
+
+	/* Maximum recursion level */
+	limits.maxreclevel = (unsigned int)scanner_opts->max_recursions;
+
+	/* Maximum mail recursion level */
+	limits.maxmailrec = (unsigned int)scanner_opts->max_mail_recursions;
+
+	/* Maximum compression ratio */
+	limits.maxratio = (unsigned int)scanner_opts->max_compression_ratio;
+
+	/* Disable memory limit for bzip2 scanner */
+	limits.archivememlim = 0;
+
 
 	/* Set up scan options */
 #if defined(CL_SCAN_STDOPT)
@@ -605,7 +486,7 @@ static gint AVScanOPClamAVScanFile(
 	/* Scan this file */
 	status = cl_scandesc(
 	    fd,
-	    &virus_name, &size_mb,
+	    &virus_name, &scanned_blocks,
 	    d->database,
 	    &limits,
 	    options
@@ -616,7 +497,23 @@ static gint AVScanOPClamAVScanFile(
 
 	/* Count this file as scanned and how much scanned */
 	d->files_scanned++;
-	d->size_scanned_mb += size_mb;
+	d->size_scanned_blocks += (gulong)scanned_blocks;
+
+	PRINT_SCANNED(
+	    d->files_scanned,
+#ifdef CL_COUNT_PRECISION
+	    (gulong)(scanned_blocks *
+		((gfloat)CL_COUNT_PRECISION / 1024.0f)
+	    ),
+	    (gulong)((gfloat)d->size_scanned_blocks *
+		((gfloat)CL_COUNT_PRECISION / 1024.0f)
+	    )
+#else     
+#warning CL_COUNT_PRECISION not defined, assuming old ClamAV 1:1 precision.
+	    (gulong)scanned_blocks * 1l,
+	    d->size_scanned_blocks * 1l
+#endif
+	);
 
 	/* Check the scan results */
 
@@ -624,12 +521,7 @@ static gint AVScanOPClamAVScanFile(
 	if(status == CL_VIRUS)
 	{
 	    /* Detected a virus */
-	    gchar *s = g_strdup_printf(
-		"Detected virus \"%s\"",
-		virus_name
-	    );
-	    PRINT_ERR(path, s);
-	    g_free(s);
+	    PRINT_INFECTED(path, virus_name);
 
 	    d->infected_files++;	/* Count this infected file */
 
@@ -641,25 +533,107 @@ static gint AVScanOPClamAVScanFile(
 	    /* Interrupted? */
 	    if(*stop_count > 0)
 		return(4);
-	    else
-		return(0);
+
+	    if(scanner_opts->flags & SCANNER_FLAG_REPORT_ALL_OBJECTS)
+		PRINT_CLEAN(path);
+
+	    return(0);
 	}
 	/* Other error */
 	else
 	{
-	    gchar *s, *error_msg = STRDUP(cl_strerror((int)status));
+	    gchar *error_msg = STRDUP(cl_strerror((int)status));
 	    if(error_msg != NULL)
 	    {
 		*error_msg = (gchar)toupper((int)*error_msg);
-		s = g_strdup_printf(
-		    "%s",
-		    error_msg
-		);
-		PRINT_ERR(path, s);
-		g_free(s);
+		PRINT_PROBLEM(path, error_msg);
 		g_free(error_msg);
 	    }
-	    return(1);
+	    switch(status)
+	    {
+	      case CL_EMAXREC:
+		status = 57;
+		break;
+	      case CL_EMAXSIZE:
+		status = 57;
+		break;
+	      case CL_EMAXFILES:
+		status = 57;
+		break;
+	      case CL_ERAR:
+		status = 59;
+		break;
+	      case CL_EZIP:
+		status = 59;
+		break;
+	      case CL_EMALFZIP:
+		status = 51;
+		break;
+	      case CL_EGZIP:
+		status = 59;
+		break;
+	      case CL_EBZIP:
+		status = 59;
+		break;
+	      case CL_EOLE2:
+		status = 59;
+		break;
+	      case CL_EMSCOMP:
+		status = 59;
+		break;
+	      case CL_EMSCAB:
+		status = 59;
+		break;
+	      case CL_EACCES:
+		status = 55;
+		break;
+	      case CL_ENULLARG:
+		status = 2;
+		break;
+	      case CL_ETMPFILE:
+		status = 3;
+		break;
+	      case CL_EFSYNC:
+		status = 3;
+		break;
+	      case CL_EMEM:
+		status = 3;
+		break;
+	      case CL_EOPEN:
+		status = 54;
+		break;
+	      case CL_EMALFDB:
+		status = 50;
+		break;
+	      case CL_EPATSHORT:
+		status = 2;
+		break;
+	      case CL_ETMPDIR:
+		status = 3;
+		break;
+	      case CL_ECVD:
+		status = 50;
+		break;
+	      case CL_ECVDEXTR:
+		status = 50;
+		break;
+	      case CL_EMD5:
+		status = 50;
+		break;
+	      case CL_EDSIG:
+		status = 50;
+		break;
+	      case CL_EIO:
+		status = 3;
+		break;
+	      case CL_EFORMAT:
+		status = 51;
+		break;
+	      default:
+		status = 1;
+		break;
+	    }
+	    return(status);
 	}
 }
 
@@ -671,7 +645,8 @@ static gint AVScanOPClamAVScanDir(
 	const gboolean recursive,
 	const gboolean executables_only,
 	const gboolean ignore_links,
-	gint *stop_count
+	gboolean *reload, gint *stop_count,
+	scanner_opts_struct *scanner_opts
 )
 {
 	mode_t m;
@@ -690,25 +665,15 @@ static gint AVScanOPClamAVScanDir(
 	dp = opendir(path);
 	if(dp == NULL)
 	{
-	    const gint error_code = errno;
+	    const gint error_code = (gint)errno;
 	    gchar *error_msg = STRDUP(g_strerror(error_code));
 	    if(error_msg != NULL)
 	    {
-		gchar *s;
 		*error_msg = (gchar)toupper((int)*error_msg);
-		s = g_strdup_printf(
-		    "%s",
-		    error_msg
-		);
-		PRINT_ERR(path, s);
-		g_free(s);
+		PRINT_PROBLEM(path, error_msg);
 		g_free(error_msg);
 	    }
-	    else
-	    {
-		PRINT_ERR(path, "Unable to open the directory");
-	    }
-	    return(1);
+	    return(53);
 	}
 
 	/* Interrupted? */
@@ -730,42 +695,119 @@ static gint AVScanOPClamAVScanDir(
 	    s = (const gchar *)dent->d_name;
 
 	    /* Check for virus database change */
-	    if(cl_statchkdir(&d->database_stat) == 1)
+	    if(*reload || (cl_statchkdir(&d->database_stat) == 1))
 	    {
 		/* Reload the virus database */
 		struct stat stat_buf;
 		gchar *s = AVScanCopyShortenString(d->database_path, 50);
 		g_print("Reloading virus database \"%s\"...\n", s);
 		g_free(s);
+
+		/* Reset the reload marker */
+		if(*reload)
+		    *reload = FALSE;
+
+		/* Delete the existing virus database */
+		if(d->database != NULL)
+		{
+		    cl_free(d->database);
+		    d->database = NULL;
+		    d->database_entries = 0;
+		}
+
+		/* Interrupted? */
+		if(*stop_count > 0)
+		    break;
+
+		if(stat((const char *)d->database_path, &stat_buf))
+		{
+		    const gint error_code = (gint)errno;
+		    gchar *error_msg = STRDUP(g_strerror(error_code));
+		    if(error_msg != NULL)
+		    {
+			*error_msg = (gchar)toupper((int)*error_msg);
+			PRINT_PROBLEM(d->database_path, error_msg);
+			g_free(error_msg);
+		    }
+		    status = 50;
+		    break;
+		}
+
 #ifdef S_ISDIR
-		if((!stat(d->database_path, &stat_buf)) ?
-		    S_ISDIR(stat_buf.st_mode) : FALSE
-		)
+		if(S_ISDIR(stat_buf.st_mode))
 #else
 		if(FALSE)
 #endif
 		    status2 = cl_loaddbdir(
-			d->database_path, &d->database, &d->database_entries
+			d->database_path,
+			&d->database, &d->database_entries
 		    );
 		else
 		    status2 = cl_loaddb(
-			d->database_path, &d->database, &d->database_entries
+			d->database_path,
+			&d->database, &d->database_entries
 		    );
-		if(status2 || (d->database == NULL) ||
+		if((status2 != 0) || (d->database == NULL) ||
 		   (d->database_entries <= 0)
 		)
 		{
-		    g_print("Failed to reload the virus database\n");
+		    gchar *s = g_strconcat(
+			"Virus database reload error: ",
+			cl_strerror(status2),
+			NULL
+		    );
+		    PRINT_PROBLEM(d->database_path, s);
+		    g_free(s);
+		    if(d->database != NULL)
+		    {
+			cl_free(d->database);
+			d->database = NULL;
+			d->database_entries = 0;
+		    }
+		    status = 50;
 		    break;
 		}
-		if(cl_build(d->database))
+		status2 = cl_build(d->database);
+		if(status2 != 0)
 		{
-		    g_print("Failed to rebuild the virus database\n");
+		    gchar *s = g_strconcat(
+			"Virus database rebuild error: ",
+			cl_strerror(status2),
+			NULL
+		    );
+		    PRINT_PROBLEM(d->database_path, s);
+		    g_free(s);
+		    if(d->database != NULL)
+		    {
+			cl_free(d->database);
+			d->database = NULL;
+			d->database_entries = 0;
+		    }
+		    status = 50;
 		    break;
 		}
-		/* Reget virus database stats */
+		/* Reget the virus database stats */
 		cl_statfree(&d->database_stat);
-		cl_statinidir(d->database_dir, &d->database_stat);
+		memset(&d->database_stat, 0x00, sizeof(d->database_stat));
+		if(cl_statinidir(d->database_dir, &d->database_stat))
+		{
+		    PRINT_PROBLEM(
+			d->database_dir,
+			"Unable to get the virus database directory statistics"
+		    );
+		    if(d->database != NULL)
+		    {
+			cl_free(d->database);
+			d->database = NULL;
+			d->database_entries = 0;
+		    }
+		    status = 50;
+		    break;
+		}
+
+		/* Interrupted? */
+		if(*stop_count > 0)
+		    break;
 	    }
 
 	    /* Skip the current and parent directory notations */
@@ -779,7 +821,11 @@ static gint AVScanOPClamAVScanDir(
 		(const char *)path, (const char *)s
 	    ));
 	    if(child == NULL)
+	    {
+		if(status == 0)
+		    status = 3;
 		continue;
+	    }
 
 	    /* Get this object's local stats */
 	    if(lstat((const char *)child, &stat_buf))
@@ -789,14 +835,12 @@ static gint AVScanOPClamAVScanDir(
 		if(error_msg != NULL)
 		{
 		    *error_msg = (gchar)toupper((int)*error_msg);
-		    PRINT_ERR(child, error_msg);
+		    PRINT_PROBLEM(child, error_msg);
 		    g_free(error_msg);
 		}
-		else
-		{
-		    PRINT_ERR(child, "Unable to obtain local stats");
-		}
 		g_free(child);
+		if(status == 0)
+		    status = 56;
 		continue;
 	    }
 
@@ -845,14 +889,12 @@ static gint AVScanOPClamAVScanDir(
 		if(error_msg != NULL)
 		{
 		    *error_msg = (gchar)toupper((int)*error_msg);
-		    PRINT_ERR(child, error_msg);
+		    PRINT_PROBLEM(child, error_msg);
 		    g_free(error_msg);
 		}
-		else
-		{
-		    PRINT_ERR(child, "Unable to obtain stats");
-		}
 		g_free(child);
+		if(status == 0)
+		    status = 56;
 		continue;
 	    }
 
@@ -868,7 +910,8 @@ static gint AVScanOPClamAVScanDir(
 		status2 = AVScanOPClamAVScanDir(
 		    d, child,
 		    recursive, executables_only, ignore_links,
-		    stop_count
+		    reload, stop_count,
+		    scanner_opts
 		);
 		if((status2 != 0) && (status == 0))
 		    status = status2;
@@ -878,7 +921,11 @@ static gint AVScanOPClamAVScanDir(
 	((m & S_IXUSR) || (m & S_IXGRP) || (m & S_IXOTH)) : TRUE
 	    )
 	    {
-		status2 = AVScanOPClamAVScanFile(d, child, stop_count);
+		status2 = AVScanOPClamAVScanFile(
+		    d, child,
+		    stop_count,
+		    scanner_opts
+		);
 		if((status2 != 0) && (status == 0))
 		    status = status2;
 	    }
@@ -889,9 +936,12 @@ static gint AVScanOPClamAVScanDir(
 	/* Close the directory */
 	closedir(dp);
 
-	/* Need to set status to interrupted? */
-	if((status == 0) && (*stop_count > 0))
-	    status = 4;
+	/* User aborted? */
+	if(*stop_count > 0)
+	{
+	    if(status == 0)
+		status = 4;
+	}
 
 	return(status);
 }
@@ -1174,6 +1224,35 @@ gchar *AVScanGetAVEngineNameVersionString(void)
  *	initialized to 0 prior to this call. This value will be checked
  *	throughout this module's functions and if (*stop_count > 0) then
  *	it will skip all scanning related calls.
+ *
+ *	Returns:
+ *
+ *	0	Success.
+ *	1	General error or virus found.
+ *	2	Invalid value.
+ *	3	Systems error or memory allocation error.
+ *	4	User aborted (otherwise no other error).
+ *	5	User responded with "no" to a query (otherwise no
+ *		other error).
+ *	6	An identical operation is already in progress.
+ *	7	Reserved.
+ *	8	Segmentation fault.
+ *
+ *	50	Unable to load or initialize the virus database.
+ *	51	An input file has a corrupt format or a format not
+ *		consistent with its extension.
+ *	52	Unsupported object type.
+ *	53	Unable to open an input directory.
+ *	54	Unable to open an input file.
+ *	55	An error occured while reading an input file.
+ *	56	Unable to stat() or lstat() an input file.
+ *	57	Scan of object is incomplete because it would
+ *		otherwise exceed a specified limit.
+ *	58	Reserved.
+ *	59	Scanner subsystem or plugin unable to scan an input
+ *		file.
+ *
+ *	99	No virus scanner support compiled.
  */
 gint AVScanOP(
 	const gchar *db_path,
@@ -1181,16 +1260,16 @@ gint AVScanOP(
 	const gboolean recursive,
 	const gboolean executables_only,
 	const gboolean ignore_links,
-	gint *stop_count		/* Must be preinitialized and not NULL */
+	gboolean *reload, gint *stop_count,
+	scanner_opts_struct *scanner_opts
 )
 {
 	gint status = 0;
-	const gulong time_start = (gulong)time(NULL);
 #if defined(HAVE_CLAMAV)
 	ClamAVData *d;
 #endif
 
-	if(paths_list == NULL)
+	if((paths_list == NULL) || (scanner_opts == NULL))
 	{
 	    status = 2;
 	    return(status);
@@ -1199,13 +1278,14 @@ gint AVScanOP(
 	/* Interrupted? */
 	if(*stop_count > 0)
 	{
-	    status = 4;
+	    if(status == 0)
+		status = 4;
 	    return(status);
 	}
 
 #if defined(HAVE_CLAMAV)
 	/* Initialize ClamAV */
-	d = AVScanOPClamAVInit(db_path);
+	d = AVScanOPClamAVInit(db_path, &status);
 	if(d != NULL)
 	{
 	    gint status2;
@@ -1233,7 +1313,7 @@ gint AVScanOP(
 		    continue;
 
 		/* Get this object's local stats */
-		if(lstat(path, &stat_buf))
+		if(lstat((const char *)path, &stat_buf))
 		{
 		    /* Unable to get this object's local stats */
 		    const gint error_code = (gint)errno;
@@ -1241,9 +1321,11 @@ gint AVScanOP(
 		    if(error_msg != NULL)
 		    {
 			*error_msg = (gchar)toupper((int)*error_msg);
-			PRINT_ERR(path, error_msg);
+			PRINT_PROBLEM(path, error_msg);
 			g_free(error_msg);
 		    }
+		    if(status == 0)
+			status = 56;
 		    continue;
 		}
 
@@ -1262,12 +1344,12 @@ gint AVScanOP(
 			 * was explicitly specified and ignore_links
 			 * was also specified
 			 */
-			PRINT_ERR(path, "Link ignored");
+			PRINT_PROBLEM(path, "Link ignored");
 			continue;
 		    }
 
 		    /* Get this link's destination stats */
-		    if(stat(path, &stat_buf))
+		    if(stat((const char *)path, &stat_buf))
 		    {
 			/* Unable to get this link's destination stats */
 			const gint error_code = (gint)errno;
@@ -1275,9 +1357,11 @@ gint AVScanOP(
 			if(error_msg != NULL)
 			{
 			    *error_msg = (gchar)toupper((int)*error_msg);
-			    PRINT_ERR(path, error_msg);
+			    PRINT_PROBLEM(path, error_msg);
 			    g_free(error_msg);
 			}
+			if(status == 0)
+			    status = 56;
 			continue;
 		    }
 
@@ -1293,14 +1377,16 @@ gint AVScanOP(
 			status2 = AVScanOPClamAVScanDir(
 			    d, path,
 			    recursive, executables_only, ignore_links,
-			    stop_count
+			    reload, stop_count,
+			    scanner_opts
 			);
 		    }
 		    else
 		    {
 			status2 = AVScanOPClamAVScanFile(
 			    d, path,
-			    stop_count
+			    stop_count,
+			    scanner_opts
 			);
 		    }
 		}
@@ -1313,7 +1399,8 @@ gint AVScanOP(
 		    status2 = AVScanOPClamAVScanDir(
 			d, path,
 			recursive, executables_only, ignore_links,
-			stop_count
+			reload, stop_count,
+			scanner_opts
 		    );
 		/* File? */
 #ifdef S_ISREG
@@ -1324,49 +1411,42 @@ gint AVScanOP(
 		{
 		    status2 = AVScanOPClamAVScanFile(
 			d, path,
-			stop_count
+			stop_count,
+			scanner_opts
 		    );
 		}
 		else
 		{
-		    PRINT_ERR(path, "Unsupported object type");
-		    status2 = 1;	/* Unsupported object type or nothing to scan */
+		    /* Unsupported object type or nothing to scan */
+		    PRINT_PROBLEM(path, "Unsupported object type");
+		    status2 = 52;
 		}
 
 		if((status2 != 0) && (status == 0))
 		    status = status2;
 	    }
 
-	    /* Not interrupted? */
-	    if(*stop_count == 0)
-	    {
-		/* Print the final report */
-		PRINT_REPORT(
-		    time_start,
-		    (gulong)time(NULL),
-		    d->infected_files,
-		    d->size_scanned_mb,
-		    d->files_scanned
-		);
-	    }
-
 	    AVScanOPClamAVShutdown(d);
 	}
 	else
 	{
-	    status = 3;
+	    /* Unable to allocate the virus scanner library data */
+	    if(status == 0)
+		status = 1;
 	}
 #else
 	PRINT_ERR(
-	    NULL,
 	    "No AntiVirus support enabled at compile time"
 	);
 	status = 1;
 #endif
 
-	/* Need to set status to interrupted? */
-	if((status == 0) && (*stop_count > 0))
-	    status = 4;
+	/* User aborted? */
+	if(*stop_count > 0)
+	{
+	    if(status == 0)
+		status = 4;
+	}
 
 	return(status);
 }
