@@ -21,11 +21,11 @@
 #define PROG_NAME		"AVScan"
 #define PROG_NAME_FULL		"AntiVirus Scanner"
 
-#define PROG_VERSION		"1.1.2"
+#define PROG_VERSION		"1.3.1"
 
 #define PROG_VERSION_MAJOR	1
-#define PROG_VERSION_MINOR	1
-#define PROG_VERSION_RELEASE	2
+#define PROG_VERSION_MINOR	3
+#define PROG_VERSION_RELEASE	1
 
 /*
  *	Program URL:
@@ -37,21 +37,21 @@
  */
 #if defined(PROG_LANGUAGE_SPANISH)
 #define PROG_COPYRIGHT	"\
-El derecho de autor (C) 1997-2006 WolfPack Entertainment.\n\
+El derecho de autor (C) 1997-2007 WolfPack Entertainment.\n\
 Este programa es protegido por leyes internacionales de derecho de autor\n\
 y tratados, la distribucin y/o la modificacin de este software en la\n\
 infraccin de la Licencia del Pblico de GNU se prohiben estrictamente.\n\
 Violators ser procesado a la extensin ms repleta de la ley.\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
 #define PROG_COPYRIGHT  "\
-Dposer (C) 1997-2006 WolfPack Entertainment.\n\
+Dposer (C) 1997-2007 WolfPack Entertainment.\n\
 Ce programme est protg par international dpose lois et traits, \n\
 distribution et/ou de ce logiciel dans violation du GNU Public Permis\n\
 strictement est interdit. Violators sera poursuivi  l'tendue plus pleine\n\
 de la loi.\n"
 #elif defined(PROG_LANGUAGE_GERMAN)
 #define PROG_COPYRIGHT  "\
-Urheberrecht (C) 1997-2006 WolfPack Entertainment.\n\
+Urheberrecht (C) 1997-2007 WolfPack Entertainment.\n\
 Dieses Programm wird von internationalem Urheberrecht Gesetzen und\n\
 Vertrgen, Austeilung bzw. nderung dieser Software in bertretung\n\
 DER GNU ffentlichkeit Erlaubnis streng verboten wird geschtzt.\n\
@@ -59,35 +59,35 @@ Verletzer werden zum vollsten Umfang des Gesetzes strafrechtlich\n\
 verfolgt werden.\n"
 #elif defined(PROG_LANGUAGE_ITALIAN)
 #define PROG_COPYRIGHT  "\
-Il copyright (C) 1997-2006 WolfPack Entertainment.\n\
+Il copyright (C) 1997-2007 WolfPack Entertainment.\n\
 Questo programma  protetto dalle leggi di copyright internazionali e\n\
 dai trattati, la distribuzione e/o la modifica di questo software nella\n\
 violazione dello GNU la Licenza Pubblica  strettamente proibita. Il\n\
 Violators sar perseguito all'estensione la pi piena della legge.\n"
 #elif defined(PROG_LANGUAGE_DUTCH)
 #define PROG_COPYRIGHT  "\
-Auteursrecht (C) 1997-2006 WolfPack Entertainment.\n\
+Auteursrecht (C) 1997-2007 WolfPack Entertainment.\n\
 Deze programma wordt door internationale auteursrecht wetten en\n\
 verdragen, verdeling en/of wijziging van deze software in overtreding\n\
 van de GNU Publiek Vergunning strikt verboden wordt beschermd.\n\
 Overtreders zullen te de volste omvang van de wet geprocedeerd worden.\n"
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 #define PROG_COPYRIGHT  "\
-Os direitos autorais (C) 1997-2006 WolfPack Entertainment.\n\
+Os direitos autorais (C) 1997-2007 WolfPack Entertainment.\n\
 Este programa  protegido por direitos internacionais leis autorais e\n\
 tratados, modificao de e/ou de distribuio deste software em infrao\n\
 do GNU Licena Pblica precisamente  proibida. O Violators ser\n\
 processado  plena extenso da lei.\n"
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 #define PROG_COPYRIGHT  "\
-Copyright (C) 1997-2006 WolfPack Entertainment.\n\
+Copyright (C) 1997-2007 WolfPack Entertainment.\n\
 Dette programet beskytter av internasjonal copyrightlover og traktater,\n\
 distribusjon og/eller modifikasjon av denne programvaren i overtredelse\n\
 av GNU Public License er streng forbudt. Violators tiltalt til den\n\
 fulleste omfang av loven.\n"
 #else
 #define PROG_COPYRIGHT  "\
-Copyright (C) 1997-2006 WolfPack Entertainment.\n\
+Copyright (C) 1997-2007 WolfPack Entertainment.\n\
 This program is protected by international copyright laws and treaties,\n\
 distribution and/or modification of this software in violation of the\n\
 GNU Public License is strictly prohibited. Violators will be prosecuted\n\
@@ -124,6 +124,26 @@ Usage: avscan <path(s)...> [options]\n\
         --database <path>       Set the virus database to <path> (this\n\
                                 may be a file or directory).\n\
         -db <path>              Same as --database.\n\
+\n\
+        --cpu-priority <priority>  Sets the CPU priority, valid values\n\
+                                are; highest, high, medium, low,\n\
+                                lowest.\n\
+\n\
+        --report-all-objects    Specifies to report all objects that\n\
+                                have been scanned to the Results List.\n\
+        --no-report-all-objects Specifies to not report all objects\n\
+                                that have been scanned to the Results\n\
+                                List.\n\
+\n\
+        --max-files <n>         Sets the maximum number of files in an\n\
+                                archive to scan.\n\
+        --max-file-size <n>     Sets the maximum size of a file in an\n\
+                                archive to scan.\n\
+        --max-recursions <n>    Sets the maximum archive recursions.\n\
+        --max-mail-recursions <n>  Sets the maximum mail recursions.\n\
+        --max-compression-ratio <n>  Set the maximum archive\n\
+                                compression ratio to scan.\n\
+\n\
         --help                  Prints (this) help screen and exits.\n\
         --version               Prints version information and exits.\n\
 \n\
@@ -133,10 +153,28 @@ Usage: avscan <path(s)...> [options]\n\
         1       Virus detected or general error.\n\
         2       Invalid value.\n\
         3       Systems error or memory allocation error.\n\
-        4       User aborted.\n\
-        5       User responded with \"no\" to a query.\n\
+        4       User aborted (otherwise no other error).\n\
+        5       User responded with \"no\" to a query (otherwise\n\
+                no other error).\n\
         6       An identical operation is already in progress.\n\
+        7       Reserved.
         8       Segmentation fault.\n\
+\n\
+        50      Unable to load or initialize the virus database.\n\
+        51      An input file has a corrupt format or a format not\n\
+                consistent with its extension.\n\
+        52      Unsupported object type.\n\
+        53      Unable to open an input directory.\n\
+        54      Unable to open an input file.\n\
+        55      An error occured while reading an input file.\n\
+        56      Unable to stat() or lstat() an input file.\n\
+        57      Scan of object was incomplete because it would\n\
+                otherwise exceed a specified limit.\n\
+        58      Reserved.\n\
+        59      Scanner subsystem or plugin unable to scan an input\n\
+                file.\n\
+\n\
+        99      No virus scanner support compiled.\n\
 \n"
 
 
@@ -160,6 +198,8 @@ Usage: avscan <path(s)...> [options]\n\
 #define DND_INFO_TEXT_URI_LIST		1	/* "text/uri-list" */
 #define DND_INFO_STRING			2	/* "STRING" */
 
+#define DND_INFO_SCAN_ITEM		3	/* AVScan/ScanItem */
+#define DND_TARGET_SCAN_ITEM		"AVScan/ScanItem"
 
 /*
  *	List Defaults:
@@ -173,6 +213,7 @@ Usage: avscan <path(s)...> [options]\n\
  */
 #define AVSCAN_CFG_FILE			"avscan.ini"
 #define AVSCAN_SCAN_LIST_FILE		"avscan_scan_list.ini"
+#define AVSCAN_LAST_SCAN_RESULTS_LOG_FILE	"avscan.log"
 #if defined(DEBIAN)
 # define AVSCAN_DB_LOCATION		"/var/lib/clamav"
 #else
