@@ -28,14 +28,18 @@ void GUIImageBufferCopyArea(
 	guint8 *tar_data,
 	gint tar_width, gint tar_height, gint tar_bpl,
 	gint tar_x, gint tar_y,
-	const gboolean blend
+	const gboolean blend,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 );
 void GUIImageBufferResize(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 );
 void GUIImageBufferRotateCW90(
 	gint bpp,
@@ -74,10 +78,12 @@ void GUIImageBufferMirrorV(
 );
 
 void GUIImageBufferFlattenWithBG(
-	gint bpp,
+	const gint bpp,
 	guint8 *data,
-	gint width, gint height, gint bpl,
-	const guint8 *bg_color
+	const gint width, const gint height, const gint bpl,
+	const guint8 *bg_color,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 );
 
 guint8 *GUIGetRGBImage(
@@ -200,11 +206,13 @@ void GUIImageBufferCopyArea(
 	guint8 *tar_data,
 	gint tar_width, gint tar_height, gint tar_bpl,
 	gint tar_x, gint tar_y,
-	const gboolean blend
+	const gboolean blend,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 )
 {
-	gint x, y;
-	const guint8 *src_line, *src_line_end, *src_ptr, *src_end;
+	gint x, y, src_y;
+	const guint8 *src_ptr, *src_end;
 	void (*pixel_op_func)(const gint, guint8 *, const guint8 *);
 
 	if((src_width <= 0) || (src_height <= 0) ||
@@ -214,6 +222,12 @@ void GUIImageBufferCopyArea(
 	)
 	    return;
 
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb(0l, (gulong)src_height, progress_data))
+		return;
+	}
+
 	/* Calculate bytes per line values as needed */
 	if(src_bpl <= 0)
 	    src_bpl = src_width * bpp;
@@ -241,12 +255,9 @@ void GUIImageBufferCopyArea(
 	}
 
 	/* Iterate through source image */
-	for(y = tar_y,
-	    src_line = src_data,
-	    src_line_end = src_line + (src_bpl * src_height);
-	    src_line < src_line_end;
-	    y++,
-	    src_line += src_bpl
+	for(y = tar_y, src_y = 0;
+	    src_y < src_height;
+	    y++, src_y++
 	)
 	{
 	    if(y < 0)
@@ -254,8 +265,17 @@ void GUIImageBufferCopyArea(
 	    if(y >= tar_height)
 		break;
 
+	    if((progress_cb != NULL) && ((y % 10) == 0))
+	    {
+		if(progress_cb(
+		    (gulong)src_y, (gulong)src_height,
+		    progress_data
+		))
+		    return;
+	    }
+
 	    x = tar_x;
-	    src_ptr = src_line;
+	    src_ptr = src_data + (src_y * src_bpl);
 	    src_end = src_ptr + (src_width * bpp);
 	    while(src_ptr < src_end)
 	    {
@@ -285,6 +305,12 @@ void GUIImageBufferCopyArea(
 		x++;
 	    }
 	}
+
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb((gulong)src_height, (gulong)src_height, progress_data))
+		return;
+	}
 }
 
 
@@ -293,13 +319,17 @@ void GUIImageBufferCopyArea(
  *	a different size.
  */
 void GUIImageBufferResize(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 )
 {
+	gint	_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
 	const guint8 *src_ptr8;
 	guint8 *tar_ptr8;
 	const guint16 *src_ptr16;
@@ -323,11 +353,17 @@ void GUIImageBufferResize(
 	)
 	    return;
 
-	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb(0l, (gulong)tar_height, progress_data))
+		return;
+	}
+
+	/* Calculate the bytes per line values as needed */
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
 
 	/* 8 bits */
 	if(bpp == 1)
@@ -348,19 +384,26 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr8 = (const guint8 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr8 = (guint8 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr8 = *src_ptr8;
 
-
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -396,19 +439,26 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr16 = (const guint16 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr16 = (guint16 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr16 = *src_ptr16;
 
-
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -444,20 +494,28 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr8 = (const guint8 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr8 = (guint8 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr8++ = *src_ptr8++;
 		*tar_ptr8++ = *src_ptr8++;
 		*tar_ptr8 = *src_ptr8;
 
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -493,19 +551,26 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr32 = (const guint32 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr32 = (guint32 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr32 = *src_ptr32;
 
-
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -522,6 +587,12 @@ void GUIImageBufferResize(
 		}
 	    }
 	}
+
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb((gulong)tar_height, (gulong)tar_height, progress_data))
+		return;
+	}
 }
 
 /*
@@ -780,40 +851,54 @@ void GUIImageBufferMirrorV(
 }
 
 /*
- *	Combines the image data with the specified background color.
+ *	Combines the image data with the background color.
  *
- *	Only bpp=2 (Greyscale Alpha) and bpp=4 (RGBA) are supported.
+ *	The bpp specifies the bytes per pixel and this value must
+ *	be either 2 (Greyscale Alpha) or 4 (RGBA).
  */
 void GUIImageBufferFlattenWithBG(
-	gint bpp,
+	const gint bpp,
 	guint8 *data,
-	gint width, gint height, gint bpl,
-	const guint8 *bg_color
+	const gint width, const gint height, const gint bpl,
+	const guint8 *bg_color,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 )
 {
+	gint _bpl = bpl;
 	guint tar_alpha, bg_alpha;
-	guint8 *line, *line_end, *ptr, *ptr_end;
 
 	if((data == NULL) || (bg_color == NULL))
 	    return;
 
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb(0l, (gulong)height, progress_data))
+		return;
+	}
+
 	/* Calculate bytes per line value as needed */
-	if(bpl <= 0)
-	    bpl = width * bpp;
+	if(_bpl <= 0)
+	    _bpl = width * bpp;
 
 	/* Greyscale Alpha? */
 	if(bpp == 2)
 	{
-	    for(line = data,
-		line_end = line + (bpl * height);
-		line < line_end;
-		line += bpl
-	    )
-	    {
-		ptr = line;
-		ptr_end = ptr + (width * bpp);
+	    gint y;
+	    guint8 *ptr, *end;
 
-		while(ptr < ptr_end)
+	    for(y = 0; y < height; y++)
+	    {
+		if((progress_cb != NULL) && ((y % 10) == 0))
+		{
+		    if(progress_cb((gulong)y, (gulong)height, progress_data))
+			return;
+		}
+
+		ptr = data + (y * _bpl);
+		end = ptr + (width * bpp);
+
+		while(ptr < end)
 		{
 		    tar_alpha = (guint)ptr[3];
 		    if(tar_alpha == 0xff)
@@ -840,16 +925,21 @@ void GUIImageBufferFlattenWithBG(
 	/* RGBA */
 	else if(bpp == 4)
 	{
-	    for(line = data, 
-		line_end = line + (bpl * height);
-		line < line_end;
-		line += bpl
-	    )
-	    {
-		ptr = line;
-		ptr_end = ptr + (width * bpp);
+	    gint y;
+	    guint8 *ptr, *end;
 
-		while(ptr < ptr_end)
+	    for(y = 0; y < height; y++)
+	    {
+		if((progress_cb != NULL) && ((y % 10) == 0))
+		{
+		    if(progress_cb((gulong)y, (gulong)height, progress_data))
+			return;
+		}
+
+		ptr = data + (y * _bpl);
+		end = ptr + (width * bpp);
+
+		while(ptr < end)
 		{
 		    tar_alpha = (guint)ptr[3];
 		    if(tar_alpha == 0xff)
@@ -881,6 +971,12 @@ void GUIImageBufferFlattenWithBG(
 		}
 	    }
 	}
+
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb((gulong)height, (gulong)height, progress_data))
+		return;
+	}
 }
 
 
