@@ -152,10 +152,31 @@ typedef struct {
 #define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 #define OBJEXISTS(p)		(((p) != NULL) ? (!access((p), F_OK)) : FALSE)
-#define OBJEXECUTABLE(p)	(((p) != NULL) ? (!access((p), X_OK)) : FALSE)
 #define OBJREADABLE(p)		(((p) != NULL) ? (!access((p), R_OK)) : FALSE)
 
 
+static gboolean OBJEXECUTABLE(const gchar *p)
+{
+	mode_t m;
+	struct stat stat_buf;
+
+	if(stat((const char *)p, &stat_buf))
+	    return(FALSE);
+
+	m = stat_buf.st_mode;
+
+#if defined(S_IXUSR) && defined(S_IXGRP) && defined(S_IXOTH)
+	if((m & S_IXUSR) ||
+	   (m & S_IXGRP) ||
+	   (m & S_IXOTH)
+	)
+	    return(TRUE);
+#endif
+
+	return(FALSE);
+}
+
+
 /*
  *	Displays the help page specified by topic by calling the help
  *	browser.
@@ -188,12 +209,12 @@ void Help(
 
 #ifdef HAVE_EDV2
 	/* Get global data directory */
-        data_dir = EDVGetS(core->edv_ctx, EDV_CFG_PARM_DIR_GLOBAL);
+	data_dir = EDVGetS(core->edv_ctx, EDV_CFG_PARM_DIR_GLOBAL);
 #else
 	data_dir = NULL;
 #endif
 	if(STRISEMPTY(data_dir))
-            return;
+	    return;
 
 	/* Get help file based on the specified topic */
 	for(i = 0, hf_ptr = NULL;
