@@ -16,14 +16,15 @@
 #include <unistd.h>
 
 #include "guiutils.h"
+#include "fprompt.h"
 #include "cdialog.h"
 #include "pdialog.h"
-#include "fprompt.h"
 #include "fb.h"
 
-#include "avscanop.h"
 #include "cfg.h"
 #include "cfg_fio.h"
+#include "scanner_opts.h"
+#include "avscanop.h"
 #include "obj.h"
 #include "objfio.h"
 #include "win.h"
@@ -48,7 +49,8 @@ static cfg_item_struct *MainCfgListNew(void);
 win_struct *MainNewAVScan(core_struct *core, gint argc, gchar **argv);
 
 
-static gint	*avscan_scan_stop_count;
+static gint		*avscan_scan_stop_count;
+static gboolean		*avscan_reload;
 
 
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
@@ -70,6 +72,14 @@ static void MainSignalCB(int s)
 {
 	switch(s)
 	{
+#ifdef SIGHUP
+	  case SIGHUP:
+	    if(avscan_reload != NULL)
+		*avscan_reload = TRUE;
+	    break;
+#endif
+
+#ifdef SIGINT
 	  case SIGINT:
 	    /* Is this process running as a scan process? */
 	    if(avscan_scan_stop_count != NULL)
@@ -93,20 +103,24 @@ static void MainSignalCB(int s)
 		exit(4);	/* User aborted */
 	    }
 	    break;
+#endif
 
+#ifdef SIGTERM
 	  case SIGTERM:
-	    exit(4);	/* User aborted */
+	    exit(4);		/* User aborted */
 	    break;
+#endif
 
+#ifdef SIGSEGV
 	  case SIGSEGV:
-	    exit(8);	/* Segmentation fault */
+	    exit(8);		/* Segmentation fault */
 	    break;
+#endif
 
-	  case SIGSTOP:
-	    break;
-
-	  case SIGCONT:
+#ifdef SIGPIPE
+	  case SIGPIPE:
 	    break;
+#endif
 	}
 }
 
@@ -150,6 +164,7 @@ static gint MainTOCB(gpointer data)
 	if((total_windows == 0) || core->close_all_windows)
 	{
 	    gtk_main_quit();
+	    core->main_toid = 0;
 	    return(FALSE);
 	}
 
@@ -192,6 +207,35 @@ static cfg_item_struct *MainCfgListNew(void)
 	    cfg_list, CFG_PARM_DB_LAST_UPDATE,
 	    0l, FALSE
 	);
+
+	CFGItemListSetValueI(
+	    cfg_list, CFG_PARM_SCAN_CPU_PRIORITY,
+	    1, FALSE
+	);
+	CFGItemListSetValueI(
+	    cfg_list, CFG_PARM_SCAN_REPORT_ALL_OBJECTS,
+	    FALSE, FALSE
+	);
+	CFGItemListSetValueL(
+	    cfg_list, CFG_PARM_SCAN_MAX_FILES,
+	    1000l, FALSE
+	);
+	CFGItemListSetValueL(
+	    cfg_list, CFG_PARM_SCAN_MAX_FILE_SIZE,
+	    10l * 1048576l, FALSE
+	);
+	CFGItemListSetValueL(
+	    cfg_list, CFG_PARM_SCAN_MAX_RECURSIONS,
+	    5l, FALSE
+	);
+	CFGItemListSetValueL(
+	    cfg_list, CFG_PARM_SCAN_MAX_MAIL_RECURSIONS,
+	    64l, FALSE
+	);
+	CFGItemListSetValueL(
+	    cfg_list, CFG_PARM_SCAN_MAX_COMPRESSION_RATIO,
+	    200l, FALSE
+	);
  
 	CFGItemListSetValueI(
 	    cfg_list, CFG_PARM_WIN_X,
@@ -286,15 +330,22 @@ int main(int argc, char *argv[])
 	core_struct *core;
 
 	/* Initialize the globals */
+	avscan_reload = NULL;
 	avscan_scan_stop_count = NULL;
 
 	/* Set up the signal callbacks */
+#ifdef SIGINT
 	signal(SIGINT, MainSignalCB);
+#endif
+#ifdef SIGTERM
 	signal(SIGTERM, MainSignalCB);
+#endif
+#ifdef SIGSEGV
 	signal(SIGSEGV, MainSignalCB);
-	signal(SIGSTOP, MainSignalCB);
-	signal(SIGCONT, MainSignalCB);
+#endif
+#ifdef SIGPIPE
 	signal(SIGPIPE, MainSignalCB);
+#endif
 
 	/* Get the current working directory */
 	cwd = g_get_current_dir();
@@ -341,16 +392,37 @@ AntiVirus Engine: %s\n",
 	    else if(!g_strcasecmp(arg, "--scan"))
 	    {
 		/* Do scan procedure and return */
+		gboolean reload = FALSE;
 		gint status, stop_count = 0;
 		gchar *db_path = NULL;
 		GList *path_list = NULL;
 		gboolean	recursive = FALSE,
 				executables_only = FALSE,
 				ignore_links = FALSE;
+		scanner_opts_struct scanner_opts;
+
+#define DELETE_LOCALS	{				\
+ if(path_list != NULL) {				\
+  g_list_foreach(path_list, (GFunc)g_free, NULL);	\
+  g_list_free(path_list);				\
+ }							\
+ g_free(db_path);					\
+}
+
 
 		/* Set the stop count pointer for the signal callback */
+		avscan_reload = &reload;
 		avscan_scan_stop_count = &stop_count;
 
+		scanner_opts.flags = 0;
+		scanner_opts.max_files = 1000l;
+		scanner_opts.max_file_size_bytes = 10l * 1048576l;
+		scanner_opts.max_recursions = 5l;
+		scanner_opts.max_mail_recursions = 64l;
+		scanner_opts.max_compression_ratio = 200l;
+		scanner_opts.cpu_priority = SCANNER_CPU_PRIORITY_HIGH;
+
+
 		/* Collect subsequent arguments */
 		for(i++; i < argc; i++)
 		{
@@ -366,12 +438,18 @@ AntiVirus Engine: %s\n",
 			i++;
 			arg = (i < argc) ? argv[i] : NULL;
 			if(arg != NULL)
+			{
 			    db_path = STRDUP(arg);
+			}
 			else
+			{
 			    g_printerr(
 				"%s: Requires argument.\n",
 				argv[i - 1]
 			    );
+			    DELETE_LOCALS
+			    return(2);
+			}
 		    }
 		    /* Recursive */
 		    else if(!g_strcasecmp(arg, "--recursive") ||
@@ -396,6 +474,159 @@ AntiVirus Engine: %s\n",
 		    {
 			ignore_links = TRUE;
 		    }
+		    /* Report All Objects */
+		    else if(!g_strcasecmp(arg, "--report-all-objects") ||
+			    !g_strcasecmp(arg, "--report_all_objects")
+		    )
+		    {
+			scanner_opts.flags |= SCANNER_FLAG_REPORT_ALL_OBJECTS;
+		    }
+		    /* Max Files */
+		    else if(!g_strcasecmp(arg, "--max-files") ||
+		            !g_strcasecmp(arg, "--max_files")
+		    )
+		    {
+			i++;
+			arg = (i < argc) ? argv[i] : NULL;
+			if(arg != NULL)
+			{
+			    scanner_opts.max_files = ATOL(arg);
+			}
+			else
+			{
+			    g_printerr(
+				"%s: Requires argument.\n",
+				argv[i - 1]
+			    );
+			    DELETE_LOCALS
+			    return(2);
+			}
+		    }
+		    /* Max File Size */
+		    else if(!g_strcasecmp(arg, "--max-file-size") ||
+		            !g_strcasecmp(arg, "--max_file_size")
+		    )
+		    {
+			i++;
+			arg = (i < argc) ? argv[i] : NULL;
+			if(arg != NULL)
+			{
+			    scanner_opts.max_file_size_bytes = ATOL(arg);
+			}
+			else
+			{
+			    g_printerr(
+				"%s: Requires argument.\n",
+				argv[i - 1]
+			    );
+			    DELETE_LOCALS
+			    return(2);
+			}
+		    }
+		    /* Max Recursions */
+		    else if(!g_strcasecmp(arg, "--max-recursions") ||
+		            !g_strcasecmp(arg, "--max_recursions")
+		    )
+		    {
+			i++;
+			arg = (i < argc) ? argv[i] : NULL;
+			if(arg != NULL)
+			{
+			    scanner_opts.max_recursions = ATOL(arg);
+			}
+			else
+			{
+			    g_printerr(
+				"%s: Requires argument.\n",
+				argv[i - 1]
+			    );
+			    DELETE_LOCALS
+			    return(2);
+			}
+		    }
+		    /* Max Mail Recursions */
+		    else if(!g_strcasecmp(arg, "--max-mail-recursions") ||
+		            !g_strcasecmp(arg, "--max_mail_recursions")
+		    )
+		    {
+			i++;
+			arg = (i < argc) ? argv[i] : NULL;
+			if(arg != NULL)
+			{
+			    scanner_opts.max_mail_recursions = ATOL(arg);
+			}
+			else
+			{
+			    g_printerr(
+				"%s: Requires argument.\n",
+				argv[i - 1]
+			    );
+			    DELETE_LOCALS
+			    return(2);
+			}
+		    }
+		    /* Max Compression Ratio */
+		    else if(!g_strcasecmp(arg, "--max-compression-ratio") ||
+		            !g_strcasecmp(arg, "--max_compression_ratio")
+		    )
+		    {
+			i++;
+			arg = (i < argc) ? argv[i] : NULL;
+			if(arg != NULL)
+			{
+			    scanner_opts.max_compression_ratio = ATOL(arg);
+			}
+			else
+			{
+			    g_printerr(
+				"%s: Requires argument.\n",
+				argv[i - 1]
+			    );
+			    DELETE_LOCALS
+			    return(2);
+			}
+		    }
+		    /* CPU Priority */
+		    else if(!g_strcasecmp(arg, "--cpu-priority") ||
+		            !g_strcasecmp(arg, "--cpu_priority")
+		    )
+		    {
+			i++;
+			arg = (i < argc) ? argv[i] : NULL;
+			if(arg != NULL)
+			{
+			    scanner_cpu_priority p = SCANNER_CPU_PRIORITY_LOWEST;
+			    if(!g_strcasecmp(arg, "highest"))
+				p = SCANNER_CPU_PRIORITY_HIGHEST;
+			    else if(!g_strcasecmp(arg, "high"))
+				p = SCANNER_CPU_PRIORITY_HIGH;
+			    else if(!g_strcasecmp(arg, "medium"))
+				p = SCANNER_CPU_PRIORITY_MEDIUM;
+			    else if(!g_strcasecmp(arg, "low"))
+				p = SCANNER_CPU_PRIORITY_LOW;
+			    else if(!g_strcasecmp(arg, "lowest"))
+				p = SCANNER_CPU_PRIORITY_LOWEST;
+			    else
+			    {
+				g_printerr(
+"%s: Invalid value, valid values are; highest, high, medium, low, lowest.\n",
+				    argv[i - 1]
+				);
+				DELETE_LOCALS
+				return(2);
+			    }
+			    scanner_opts.cpu_priority = p;
+			}
+			else
+			{
+			    g_printerr(
+				"%s: Requires argument.\n",
+				argv[i - 1]
+			    );
+			    DELETE_LOCALS
+			    return(2);
+			}
+		    }
 		    /* All else assume path */
 		    else if((*arg != '-') && (*arg != '+'))
 		    {
@@ -404,30 +635,102 @@ AntiVirus Engine: %s\n",
 			    g_strconcat(cwd, G_DIR_SEPARATOR_S, arg, NULL);
 			path_list = g_list_append(path_list, path);
 		    }
+		    /* Unsupported argument? */
+		    else if((*arg == '-') || (*arg == '+'))
+		    {
+			g_printerr(
+			    "%s: Unsupported argument.\n",
+			    arg
+			);
+			DELETE_LOCALS
+			return(2);
+		    }
 		}
 
-		/* Do scan procedure */
+		/* Do the scan procedure */
 		status = AVScanOP(
 		    db_path,		/* Database Path */
 		    path_list,		/* Paths to objects to scan */
-		    recursive,		/* Recursive */
-		    executables_only,	/* Executables Only */
-		    ignore_links,	/* Ignore Links */
-		    &stop_count
+		    recursive,
+		    executables_only,
+		    ignore_links,
+		    avscan_reload, avscan_scan_stop_count,
+		    &scanner_opts
 		);
 
 		/* Unset the stop count pointer */
+		avscan_reload = NULL;
 		avscan_scan_stop_count = NULL;
 
-		g_list_foreach(path_list, (GFunc)g_free, NULL);
-		g_list_free(path_list);
-
-		g_free(db_path);
+		DELETE_LOCALS
 
 		return(status);
+#undef DELETE_LOCALS
 	    }
 	}
 
+#ifdef HAVE_EDV2
+#define DELETE_CORE_LIBS	{			\
+ if(core != NULL) {					\
+  /* Endeavour Mark II context */			\
+  if(core->edv_ctx != NULL) {				\
+   EDVContextDelete(core->edv_ctx);			\
+   core->edv_ctx = NULL;				\
+  }							\
+ }							\
+}
+#else
+#define DELETE_CORE_LIBS	{ }
+#endif
+
+#define DELETE_CORE_SHUTDOWN	{			\
+							\
+ if(core != NULL) {					\
+  /* Windows */						\
+  if(core->win != NULL) {				\
+   g_list_foreach(core->win, (GFunc)WinDelete, NULL);	\
+   g_list_free(core->win);				\
+   core->win = NULL;					\
+  }							\
+							\
+  /* Cursors */						\
+  GDK_CURSOR_DESTROY(core->busy_cur);			\
+  GDK_CURSOR_DESTROY(core->passive_busy_cur);		\
+							\
+  /* Pixmaps */						\
+  GDK_PIXMAP_UNREF(core->scan_item_pixmap);		\
+  GDK_BITMAP_UNREF(core->scan_item_mask);		\
+  GDK_PIXMAP_UNREF(core->file_infected_pixmap);		\
+  GDK_BITMAP_UNREF(core->file_infected_mask);		\
+  GDK_PIXMAP_UNREF(core->folder_closed_pixmap);		\
+  GDK_BITMAP_UNREF(core->folder_closed_mask);		\
+  GDK_PIXMAP_UNREF(core->folder_opened_pixmap);		\
+  GDK_BITMAP_UNREF(core->folder_opened_mask);		\
+  GDK_PIXMAP_UNREF(core->db_item_pixmap);		\
+  GDK_BITMAP_UNREF(core->db_item_mask);			\
+							\
+  /* Configuration List */				\
+  CFGItemListDeleteList(core->cfg_list);		\
+							\
+  DELETE_CORE_LIBS					\
+							\
+  g_free(core->prog_name);				\
+  g_free(core->prog_path);				\
+  g_free(core->prog_data_dir);				\
+							\
+  g_free(core->title);					\
+  g_free(core->geometry);				\
+							\
+  g_free(core);						\
+  core = NULL;						\
+ }							\
+							\
+ /* Shutdown dialogs */					\
+ PDialogShutdown();					\
+ FileBrowserShutdown();					\
+ FPromptShutdown();					\
+ CDialogShutdown();					\
+}
 
 	/* Initialize GTK as needed */
 	if(!initialized_gtk)
@@ -450,24 +753,33 @@ AntiVirus Engine: %s\n",
 
 	/* Initialize dialogs */
 	CDialogInit();
-	PDialogInit();
 	FPromptInit();
 	FileBrowserInit();
+	PDialogInit();
 
 
 	/* Begin creating the core */
-	core = CORE(
-	    g_malloc0(sizeof(core_struct))
-	);
+	core = CORE(g_malloc0(sizeof(core_struct)));
+	if(core == NULL)
+	{
+	    DELETE_CORE_SHUTDOWN
+	    return(3);
+	}
 
 	/* Configuration List */
 	core->cfg_list = cfg_list = MainCfgListNew();
 
+
 #ifdef HAVE_EDV2
 	/* Endeavour Mark II context */
 	core->edv_ctx = ctx = EDVContextNew();
 #endif
 
+	core->target_scan_item_atom = gdk_atom_intern(
+	    DND_TARGET_SCAN_ITEM,
+	    FALSE
+	);
+
 	/* Pixmaps */
 	core->scan_item_pixmap = GDK_PIXMAP_NEW_FROM_XPM_DATA(
 	    &core->scan_item_mask,
@@ -577,7 +889,7 @@ AntiVirus Engine: %s\n",
 	core->prog_data_dir = NULL;
 
 #ifdef HAVE_EDV2
-	/* Load Endeavour configuration from file */
+	/* Load the Endeavour configuration from file */
 	EDVContextLoadConfigurationFile(ctx, NULL);
 #endif
 
@@ -618,6 +930,13 @@ AntiVirus Engine: %s\n",
 	{
 	    GList *paths_list = NULL;
 
+#define DELETE_LOCALS	{				\
+ if(paths_list != NULL) {				\
+  g_list_foreach(paths_list, (GFunc)g_free, NULL);	\
+  g_list_free(paths_list);				\
+ }							\
+}
+
 	    /* Parse the arguments */
 	    for(i = 1; i < argc; i++)
 	    {
@@ -633,15 +952,22 @@ AntiVirus Engine: %s\n",
 		    i++;
 		    arg = (i < argc) ? argv[i] : NULL;
 		    if(arg != NULL)
+		    {
 			CFGItemListSetValueS(
 			    cfg_list, CFG_PARM_DB_LOCATION,
 			    arg, FALSE
 			);
+		    }
 		    else
+		    {
 			g_printerr(
 			    "%s: Requires argument.\n",
 			    argv[i - 1]
 			);
+			DELETE_LOCALS
+			DELETE_CORE_SHUTDOWN
+			return(2);
+		    }
 		}
 		/* Recursive */
 		else if(!g_strcasecmp(arg, "--recursive") ||
@@ -675,7 +1001,198 @@ AntiVirus Engine: %s\n",
 			TRUE
 		    );
 		}
-		/* Skip GUI arguments that have a an argument */
+		/* Report All Objects */
+		else if(!g_strcasecmp(arg, "--report-all-objects") ||
+		        !g_strcasecmp(arg, "--report_all_objects")
+		)
+		{
+		    CFGItemListSetValueL(
+			cfg_list, CFG_PARM_SCAN_REPORT_ALL_OBJECTS,
+			TRUE, FALSE
+		    );
+		}
+		/* Do Not Report All Objects */
+		else if(!g_strcasecmp(arg, "--no-report-all-objects") ||
+		        !g_strcasecmp(arg, "--no-report_all_objects")
+		)
+		{
+		    CFGItemListSetValueL(
+			cfg_list, CFG_PARM_SCAN_REPORT_ALL_OBJECTS,
+			FALSE, FALSE
+		    );
+		}
+		/* Max Files */
+		else if(!g_strcasecmp(arg, "--max-files") ||
+		        !g_strcasecmp(arg, "--max_files")
+		)
+		{
+		    i++;
+		    arg = (i < argc) ? argv[i] : NULL;
+		    if(arg != NULL)
+		    {
+			CFGItemListSetValueL(
+			    cfg_list, CFG_PARM_SCAN_MAX_FILES,
+			    ATOL(arg), FALSE
+			);
+		    }
+		    else
+		    {
+			g_printerr(
+			    "%s: Requires argument.\n",
+			    argv[i - 1]
+			);
+			DELETE_LOCALS
+			DELETE_CORE_SHUTDOWN
+			return(2);
+		    }
+		}
+		/* Max File Size */
+		else if(!g_strcasecmp(arg, "--max-file-size") ||
+		        !g_strcasecmp(arg, "--max_file_size")
+		)
+		{
+		    i++;
+		    arg = (i < argc) ? argv[i] : NULL;
+		    if(arg != NULL)
+		    {
+			CFGItemListSetValueL(
+			    cfg_list, CFG_PARM_SCAN_MAX_FILE_SIZE,
+			    ATOL(arg), FALSE
+			);
+		    }
+		    else
+		    {
+			g_printerr(
+			    "%s: Requires argument.\n",
+			    argv[i - 1]
+			);
+			DELETE_LOCALS
+			DELETE_CORE_SHUTDOWN
+			return(2);
+		    }
+		}
+		/* Max Recursions */
+		else if(!g_strcasecmp(arg, "--max-recursions") ||
+		        !g_strcasecmp(arg, "--max_recursions")
+		)
+		{
+		    i++;
+		    arg = (i < argc) ? argv[i] : NULL;
+		    if(arg != NULL)
+		    {
+			CFGItemListSetValueL(
+			    cfg_list, CFG_PARM_SCAN_MAX_RECURSIONS,
+			    ATOL(arg), FALSE
+			);
+		    }
+		    else
+		    {
+			g_printerr(
+			    "%s: Requires argument.\n",
+			    argv[i - 1]
+			);
+			DELETE_LOCALS
+			DELETE_CORE_SHUTDOWN
+			return(2);
+		    }
+		}
+		/* Max Mail Recursions */
+		else if(!g_strcasecmp(arg, "--max-mail-recursions") ||
+		        !g_strcasecmp(arg, "--max_mail_recursions")
+		)
+		{
+		    i++;
+		    arg = (i < argc) ? argv[i] : NULL;
+		    if(arg != NULL)
+		    {
+			CFGItemListSetValueL(
+			    cfg_list, CFG_PARM_SCAN_MAX_MAIL_RECURSIONS,
+			    ATOL(arg), FALSE
+			);
+		    }
+		    else
+		    {
+			g_printerr(
+			    "%s: Requires argument.\n",
+			    argv[i - 1]
+			);
+			DELETE_LOCALS
+			DELETE_CORE_SHUTDOWN
+			return(2);
+		    }
+		}
+		/* Max Compression Ratio */
+		else if(!g_strcasecmp(arg, "--max-compression-ratio") ||
+		        !g_strcasecmp(arg, "--max_compression_ratio")
+		)
+		{
+		    i++;
+		    arg = (i < argc) ? argv[i] : NULL;
+		    if(arg != NULL)
+		    {
+			CFGItemListSetValueL(
+			    cfg_list, CFG_PARM_SCAN_MAX_COMPRESSION_RATIO,
+			    ATOL(arg), FALSE
+			);
+		    }
+		    else
+		    {
+			g_printerr(
+			    "%s: Requires argument.\n",
+			    argv[i - 1]
+			);
+			DELETE_LOCALS
+			DELETE_CORE_SHUTDOWN
+			return(2);
+		    }
+		}
+		/* CPU Priority */
+		else if(!g_strcasecmp(arg, "--cpu-priority") ||
+		        !g_strcasecmp(arg, "--cpu_priority")
+		)
+		{
+		    i++;
+		    arg = (i < argc) ? argv[i] : NULL;
+		    if(arg != NULL)
+		    {
+			scanner_cpu_priority p = SCANNER_CPU_PRIORITY_LOWEST;
+			if(!g_strcasecmp(arg, "highest"))
+			    p = SCANNER_CPU_PRIORITY_HIGHEST;
+			else if(!g_strcasecmp(arg, "high"))
+			    p = SCANNER_CPU_PRIORITY_HIGH;
+			else if(!g_strcasecmp(arg, "medium"))
+			    p = SCANNER_CPU_PRIORITY_MEDIUM;
+			else if(!g_strcasecmp(arg, "low"))
+			    p = SCANNER_CPU_PRIORITY_LOW;
+			else if(!g_strcasecmp(arg, "lowest"))
+			    p = SCANNER_CPU_PRIORITY_LOWEST;
+			else
+			{
+			    g_printerr(
+"%s: Invalid value, valid values are; highest, high, medium, low, lowest.\n",
+				argv[i - 1]
+			    );
+			    DELETE_LOCALS
+			    DELETE_CORE_SHUTDOWN
+			    return(2);
+			}
+			CFGItemListSetValueI(
+			    cfg_list, CFG_PARM_SCAN_CPU_PRIORITY,
+			    (gint)p, FALSE
+			);
+		    }
+		    else
+		    {
+			g_printerr(
+			    "%s: Requires argument.\n",
+			    argv[i - 1]
+			);
+			DELETE_LOCALS
+			DELETE_CORE_SHUTDOWN
+			return(2);
+		    }
+		}
+		/* Skip GUI arguments that have an argument */
 		else if(!g_strcasecmp(arg, "--geometry") ||
 			!g_strcasecmp(arg, "-geometry") ||
 			!g_strcasecmp(arg, "--fn") ||
@@ -739,14 +1256,20 @@ AntiVirus Engine: %s\n",
 		g_free(location);
 
 		WinStartCB(win->start_btn, win);
-
-		g_list_foreach(paths_list, (GFunc)g_free, NULL);
-		g_list_free(paths_list);
 	    }
+
+	    DELETE_LOCALS
+
+#undef DELETE_LOCALS
 	}
 
-	/* Set main timeout callback */
-	gtk_timeout_add(1000l, MainTOCB, core);
+	/* Set the main timeout callback */
+	core->main_toid = gtk_timeout_add(
+	    1000l,
+	    MainTOCB, core
+	);
+
+	/* Push the main GTK+ loop */
 	gtk_main();
 
 
@@ -760,52 +1283,10 @@ AntiVirus Engine: %s\n",
 	CFGFileSave(path, cfg_list);
 	g_free(path);
 
-
-	/* Begin deleting core */
-	g_list_foreach(core->win, (GFunc)WinDelete, NULL);
-	g_list_free(core->win);
-	core->win = NULL;
-
-	/* Cursors */
-	GDK_CURSOR_DESTROY(core->busy_cur);
-	GDK_CURSOR_DESTROY(core->passive_busy_cur);
-
-	/* Pixmaps */
-	GDK_PIXMAP_UNREF(core->scan_item_pixmap);
-	GDK_BITMAP_UNREF(core->scan_item_mask);
-	GDK_PIXMAP_UNREF(core->file_infected_pixmap);
-	GDK_BITMAP_UNREF(core->file_infected_mask);
-	GDK_PIXMAP_UNREF(core->folder_closed_pixmap);
-	GDK_BITMAP_UNREF(core->folder_closed_mask);
-	GDK_PIXMAP_UNREF(core->folder_opened_pixmap);
-	GDK_BITMAP_UNREF(core->folder_opened_mask);
-	GDK_PIXMAP_UNREF(core->db_item_pixmap);
-	GDK_BITMAP_UNREF(core->db_item_mask);
-
-	/* Configuration List */
-	CFGItemListDeleteList(core->cfg_list);
-
-#ifdef HAVE_EDV2
-	/* Endeavour Mark II context */
-	EDVContextDelete(core->edv_ctx);
-#endif
-
-	g_free(core->prog_name);
-	g_free(core->prog_path);
-	g_free(core->prog_data_dir);
-
-	g_free(core->title);
-	g_free(core->geometry);
-
-	g_free(core);
-	core = NULL;
-
-
-	/* Shutdown dialogs */
-	FileBrowserShutdown();
-	FPromptShutdown();
-	PDialogShutdown();
-	CDialogShutdown();
+	DELETE_CORE_SHUTDOWN
 
 	return(0);
+
+#undef DELETE_CORE_SHUTDOWN
+#undef DELETE_CORE_LIBS
 }
