@@ -1226,6 +1226,10 @@ gint PDialogInit(void)
 		GDK_HINT_BASE_SIZE | GDK_HINT_RESIZE_INC
 	    );
 	}
+	gtk_widget_add_events(
+	    w,
+	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
+	);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "delete_event",
 	    GTK_SIGNAL_FUNC(PDialogCloseCB), d
