@@ -22,9 +22,6 @@
 #include "core.h"
 #include "cfglist.h"
 
-#include "images/icon_avscan_32x32.xpm"
-#include "images/icon_biohazard_32x32.xpm"
-
 
 static const gchar *SEEK_PAST_ARG(const gchar *s);
 static const gchar *SEEK_PAST_SPACES(const gchar *s);
@@ -37,6 +34,7 @@ static gchar *SHORTEN_STRING(const gchar *s, const gint m);
 
 void WinScanListItemDestroyCB(gpointer data);
 void WinResultsListItemDestroyCB(gpointer data);
+void WinDBListItemDestroyCB(gpointer data);
 
 gint WinEventCB(
 	GtkWidget *widget, GdkEvent *event, gpointer data
@@ -64,6 +62,7 @@ void WinNotebookSwitchPageCB(
 	gpointer data
 );
 
+void WinLocationEntryChangedCB(GtkWidget *widget, gpointer data);
 gint WinEntryCompletePathKeyCB(
 	GtkWidget *widget, GdkEventKey *key, gpointer data
 );
@@ -108,7 +107,13 @@ void WinDBFindEntryCB(GtkWidget *widget, gpointer data);
 gint WinDBTreeLoadStartIdleCB(gpointer data);
 
 gint WinScanMonitorTOCB(gpointer data);
-gint WinUpdateMonitorTOCB(gpointer data);
+gint WinDBUpdateNetMonitorTOCB(gpointer data);
+
+
+gint WinPatternDlgDeleteEventCB(
+	GtkWidget *widget, GdkEvent *event, gpointer data
+);
+void WinPatternDlgCloseCB(GtkWidget *widget, gpointer data);
 
 
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
@@ -235,7 +240,7 @@ static gchar *SHORTEN_STRING(const gchar *s, const gint m)
 
 
 /*
- *	Win Scan List item destroy signal callback.
+ *	Scan List item destroy signal callback.
  */
 void WinScanListItemDestroyCB(gpointer data)
 {
@@ -243,13 +248,25 @@ void WinScanListItemDestroyCB(gpointer data)
 }
 
 /*
- *	Win Results List item destroy signal callback.
+ *	Results List item destroy signal callback.
  */
 void WinResultsListItemDestroyCB(gpointer data)
 {
 	ObjDelete(OBJ(data));
 }
 
+/*
+ *	Database List item destroy signal callback.
+ */
+void WinDBListItemDestroyCB(gpointer data)
+{
+	win_db_item_struct *d = WIN_DB_ITEM(data);
+	if(d == NULL)
+	    return;
+
+	g_free(d->name);
+	g_free(d);
+}
 
 /*
  *	Win toplevel GtkWindow event signal callback.
@@ -276,7 +293,7 @@ gint WinEventCB(
 	     */
 	    if(WinScanProcessIsRunning(win) ||
 	       (win->db_ctree_load_idleid > 0) ||
-	       WinUpdateProcessIsRunning(win)
+	       WinDBUpdateNetProcessIsRunning(win)
 	    )
 		WinStopCB(win->stop_mi, win);
 	    else
@@ -414,11 +431,31 @@ void WinNotebookSwitchPageCB(
 		WinDBQueueUpdate(win);
 		break;
 	    }
+	    WinUpdateTitle(win);
 	    WinUpdate(win);
 	}
 }
 
 
+/*
+ *	Location GtkEntry "changed" signal callback.
+ */
+void WinLocationEntryChangedCB(GtkWidget *widget, gpointer data)
+{
+	win_struct *win = WIN(data);
+	if((widget == NULL) || (win == NULL))
+	    return;
+
+	if(win->freeze_count > 0)
+	    return;
+
+	win->freeze_count++;
+
+	WinUpdate(win);
+
+	win->freeze_count--;
+}
+
 /*
  *	Entry complete path "key_press_event" or "key_release_event"
  *	signal callback.
@@ -537,13 +574,13 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 	    button = (GdkEventButton *)event;
 	    switch(button->button)
 	    {
-	      case 1:
+	      case GDK_BUTTON1:
 		break;
 
-	      case 2:
+	      case GDK_BUTTON2:
 		break;
 
-	      case 3:
+	      case GDK_BUTTON3:
 		if(widget == win->scan_clist)
 		{
 		    GtkMenu *menu = GTK_MENU(win->scan_list_menu);
@@ -562,6 +599,15 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 			button->button, button->time
 		    );
 		}
+		else if(widget == win->db_ctree)
+		{
+		    GtkMenu *menu = GTK_MENU(win->db_list_menu);
+		    gtk_menu_popup(
+			menu, NULL, NULL,
+			NULL, NULL,
+			button->button, button->time
+		    );
+		}
 		status = TRUE;
 		break;
 	    }
@@ -571,11 +617,19 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 	    button = (GdkEventButton *)event;
 	    switch(button->button)
 	    {
-	      case 1:
+	      case GDK_BUTTON1:
 		if(widget == win->scan_clist)
 		{
 		    WinStartCB(NULL, win);
 		}
+		else if(widget == win->results_clist)
+		{
+
+		}
+		else if(widget == win->db_ctree)
+		{
+		    WinDBPatternDetailsCB(NULL, win);
+		}
 		break;
 	    }
 	    break;
@@ -687,7 +741,7 @@ void WinListSelectRowCB(
 	    );
 	    if(obj != NULL)
 	    {
-		gchar *buf;
+		gchar *msg;
 		const gint nobjs = g_list_length(clist->selection);
  
 		/* Update scan settings */
@@ -695,19 +749,19 @@ void WinListSelectRowCB(
 
 		/* Update status bar message */
 		if(nobjs > 1)
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"%i scan items selected",
 			nobjs
 		    );
 		else if(!STRISEMPTY(obj->name))
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"Scan item \"%s\" selected",
 			obj->name
 		    );
 		else
-		    buf = STRDUP("Scan item (Untitled) selected");
-		WinStatusMessage(win, buf, FALSE);
-		g_free(buf);
+		    msg = STRDUP("Scan item (Untitled) selected");
+		WinStatusMessage(win, msg, FALSE);
+		g_free(msg);
 	    }
 
 	    WinListDNDSetIcon(clist, row, column);
@@ -721,19 +775,19 @@ void WinListSelectRowCB(
 	    );
 	    if((obj != NULL) ? !STRISEMPTY(obj->path) : FALSE)
 	    {
-		gchar *buf;
+		gchar *msg;
 		const gint nobjs = g_list_length(clist->selection);
 		const gchar	*path = obj->path,
 				*name = GET_NAME_FROM_PATH(path);
 		struct stat stat_buf;
 
 		if(nobjs > 1)
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"%i objects selected",
 			nobjs
 		    );
 		else if(lstat(path, &stat_buf))
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"Object \"%s\" selected",
 			name
 		    );
@@ -753,11 +807,11 @@ void WinListSelectRowCB(
 			(gulong)stat_buf.st_size
 		    );
 #endif
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"File \"%s\" selected (%s byte%s)", 
 			name,
 			size_str,
-			(stat_buf.st_size == 1) ? "" : "s"
+			(stat_buf.st_size == 1l) ? "" : "s"
 		    );
 		}
 #ifdef S_ISDIR
@@ -765,7 +819,7 @@ void WinListSelectRowCB(
 #else
 		else if(FALSE)
 #endif
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"Directory \"%s\" selected",
 			name
 		    );
@@ -774,18 +828,18 @@ void WinListSelectRowCB(
 #else
 		else if(FALSE)
 #endif
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"Link \"%s\" selected",
 			name
 		    );
 		else
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 			"Object \"%s\" selected",
 			name
 		    );
 
-		WinStatusMessage(win, buf, FALSE);
-		g_free(buf);
+		WinStatusMessage(win, msg, FALSE);
+		g_free(msg);
 	    }
 	    WinListDNDSetIcon(clist, row, column);
 	    WinUpdate(win);
@@ -856,6 +910,36 @@ void WinTreeSelectRowCB(
 	/* Virus Database Tree */
 	if(ctree == (GtkCTree *)win->db_ctree)
 	{
+	    gchar *msg;
+	    win_db_item_struct *d = WIN_DB_ITEM(
+		gtk_ctree_node_get_row_data(ctree, node)
+	    );
+	    if((d != NULL) ? (d->name != NULL) : FALSE)
+	    {
+		switch(d->type)
+		{
+		  case WIN_DB_ITEM_TYPE_PATTERN:
+		    msg = g_strdup_printf(
+			"Pattern \"%s\" selected",
+			d->name
+		    );
+		    break;
+		  case WIN_DB_ITEM_TYPE_FOLDER:
+		    msg = g_strdup_printf(
+			"Folder \"%s\" selected",
+			d->name
+		    );
+		    break;
+		  default:
+		    msg = STRDUP("");
+		    break;
+		}
+	    }
+	    else
+		msg = STRDUP("");
+	    WinStatusMessage(win, msg, FALSE);
+	    g_free(msg);
+
 	    WinUpdate(win);
 	}
 }
@@ -1055,21 +1139,21 @@ void WinDBFindEntryCB(GtkWidget *widget, gpointer data)
 	}
 	else
 	{
-	    gchar *buf = g_strdup_printf(
+	    gchar *msg = g_strdup_printf(
 		"The text \"%s\" was not found",
 		needle
 	    );
 	    CDialogSetTransientFor(toplevel);
 	    CDialogGetResponse(
 		"Find Results",
-		buf,
+		msg,
 		NULL,
 		CDIALOG_ICON_SEARCH,
 		CDIALOG_BTNFLAG_OK,
 		CDIALOG_BTNFLAG_OK
 	    );
 	    CDialogSetTransientFor(NULL);
-	    g_free(buf);
+	    g_free(msg);
 	}
 
 	g_free(needle);
@@ -1085,7 +1169,7 @@ gint WinDBTreeLoadStartIdleCB(gpointer data)
 	    return(FALSE);
 
 	/* Update the Virus Database Tree as needed */
-	if(win->db_root_node == NULL)
+	if(win->db == NULL)
 	{
 	    WinDBUpdate(win, NULL, TRUE);
 	}
@@ -1107,36 +1191,31 @@ gint WinScanMonitorTOCB(gpointer data)
 	GtkWidget *toplevel;
 	GtkCList *clist;
 	core_struct *core;
+	win_scan_context_struct *scan_ctx;
 	win_struct *win = WIN(data);
 	if(win == NULL)
 	    return(FALSE);
 
-	p = win->scan_pid;
 	toplevel = win->toplevel;
 	clist = (GtkCList *)win->results_clist;
 	core = CORE(win->core);
+	scan_ctx = win->scan_context;
+	p = scan_ctx->pid;
 
 	/* User clicked on the stop button? */
 	if(win->stop_count > 0)
 	{
-	    gint new_row;
-
 	    /* Stop scanning
 	     *
 	     * Mark that this timeout is no longer being called
 	     */
-	    win->scan_toid = 0;
-
-	    new_row = WinResultsListAppend(win, NULL, "Interrupted");
-	    if(new_row > -1)
-	    {
-		gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
-	    }
+	    scan_ctx->toid = 0;
 
 	    /* Tell the scanning process to stop scanning, delete
 	     * all the tempory output files, and clean up
 	     */
 	    WinScanProcessStop(win);
+	    WinResultsStatusUpdate(win, "Scan Interrupted");
 
 	    WinStatusMessage(
 		win,
@@ -1151,73 +1230,30 @@ gint WinScanMonitorTOCB(gpointer data)
 	fp = win->stderr_fp;
 	if(fp != NULL)
 	{
-	    /* Format for each line from stderr is either:
-	     *
-	     *     "<path>" <virus|problem>
-	     *
-	     * or:
-	     *
-	     *     <problem>
-	     *
-	     * Example:
-	     *
-	     *     "/home/ftp/incoming/runme" Some virus detected!
-	     *
-	     *     The virus engine is out of date, upgrade!
+	    /* Anything from stderr will be added as a string to
+	     * the virus/problem column of the results list
 	     */
 	    gint new_row;
-	    gchar *s_end, *path, *virus_name;
-	    const gchar *s;
+	    gchar *s;
 
-	    while(fgets(buf, sizeof(buf), fp) != NULL)
+	    while(fgets((char *)buf, sizeof(buf), fp) != NULL)
 	    {
 		buf[sizeof(buf) - 1] = '\0';
 
-		s_end = strpbrk(buf, "\n\r");
-		if(s_end != NULL)
-		    *s_end = '\0';
+		s = (gchar *)strpbrk((char *)buf, "\n\r");
+		if(s != NULL)
+		    *s = '\0';
 
-		path = NULL;			/* Path */
-		virus_name = NULL;		/* Virus|problem */
+		/* Seek past any spaces */
+		s = buf;
+		while(ISBLANK(*s))
+		    s++;
 
-		/* Seek past initial spaces */
-		s = SEEK_PAST_SPACES(buf);
-
-		/* Check format, if the first character is a "
-		 * then it implies the format is:
-		 *
-		 *     "path" <virus|problem>
-		 */
-		if(*s == '"')
-		{
-		    s++;	/* Seek past initial '"' character */
-
-		    /* Get the path and increment s past the next '"'
-		     * character
-		     */
-		    s = GET_ARG(s, '"', &path);
-
-		    s = SEEK_PAST_SPACES(s);
-
-		    /* Get the virus/problem */
-		    virus_name = STRDUP(s);
-		}
-		else
-		{
-		    /* No path, just get the whole line as the virus
-		     * name
-		     */
-		    virus_name = STRDUP(s);
-		}
-
-		new_row = WinResultsListAppend(win, path, virus_name);
+		new_row = WinResultsListAppend(win, NULL, s, FALSE);
 		if(new_row > -1)
 		{
 		    gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
 		}
-
-		g_free(path);
-		g_free(virus_name);
 	    }
 	}
 
@@ -1228,17 +1264,17 @@ gint WinScanMonitorTOCB(gpointer data)
 	    /* Each line from stdout must have its prefix checked and
 	     * handled accordingly
 	     */
-	    gchar *s, *s2;
+	    gchar *s;
 
-	    while(fgets(buf, sizeof(buf), fp) != NULL)
+	    while(fgets((char *)buf, sizeof(buf), fp) != NULL)
 	    {
 		buf[sizeof(buf) - 1] = '\0';
 
-		s = strpbrk(buf, "\n\r");
+		s = (gchar *)strpbrk((char *)buf, "\n\r");
 		if(s != NULL)
 		    *s = '\0';
 
-		/* Seek past spaces */
+		/* Seek past any spaces */
 		s = buf;
 		while(ISBLANK(*s))
 		    s++;
@@ -1250,11 +1286,14 @@ gint WinScanMonitorTOCB(gpointer data)
 		{
 		    /* Format is:
 		     *
-		     * "<pfx>: "<path>" <files_scanned> <total_files>"
+		     * "<pfx>: "<path>" <nscanned> <total>"
 		     */
 		    const gchar *s2 = s;
-		    gchar *pfx = NULL, *path = NULL, *msg;
-		    gulong i = 0l, n = 0l;
+		    gchar	*pfx = NULL,
+				*path = NULL,
+				*shortened_path = NULL,
+				*msg;
+		    gulong nscanned = 0l, total = 0l;
 
 		    /* Get the prefix */
 		    if(s2 != NULL)
@@ -1265,24 +1304,18 @@ gint WinScanMonitorTOCB(gpointer data)
 		    /* Get the path */
 		    if(s2 != NULL)
 		    {
-			gchar *path2;
-
+			/* Get the path */
 			s2 = strchr(s2, '"');
 			if(s2 != NULL)
 			    s2++;
 			s2 = GET_ARG(s2, '"', &path);
-
-			/* Shorten the path as needed */
-			path2 = SHORTEN_STRING(path, 40);
-			g_free(path);
-			path = path2;
 		    }
 
 		    /* Get the number of files scanned */
 		    if(s2 != NULL)
 		    {
 			s2 = SEEK_PAST_SPACES(s2);
-			win->scan_files_scanned = i = (gulong)ATOL(s2);
+			scan_ctx->nscanned = nscanned = (gulong)ATOL(s2);
 		    }
 
 		    /* Get the total files */
@@ -1290,25 +1323,85 @@ gint WinScanMonitorTOCB(gpointer data)
 		    {
 			s2 = SEEK_PAST_ARG(s2);
 			s2 = SEEK_PAST_SPACES(s2);
-			win->scan_files_total = n = (gulong)ATOL(s2);
+			scan_ctx->total = total = (gulong)ATOL(s2);
 		    }
 
+		    WinUpdateTitle(win);
+
+		    /* Update the results status */
+		    shortened_path = SHORTEN_STRING(path, 27);
 		    msg = g_strdup_printf(
 			"%s (%ld/%ld): %s",
-			pfx, i + 1l, n, path
+			pfx, nscanned + 1l, total, shortened_path
 		    );
+		    g_free(shortened_path);
+		    WinResultsStatusUpdate(win, msg);
+		    g_free(msg);
+
+		    /* Update the status bar message */
+		    shortened_path = SHORTEN_STRING(path, 45);
+		    msg = g_strdup_printf(
+			"%s (%ld/%ld): %s",
+			pfx, nscanned + 1l, total, shortened_path
+		    );
+		    g_free(shortened_path);
 		    WinStatusMessage(win, msg, FALSE);
 		    g_free(msg);
 
+		    /* Update the status bar progress */
 		    WinStatusProgress(
 			win,
-			(n > 0l) ? ((gfloat)i / (gfloat)n) : 0.0f,
+			(scan_ctx->total > 0l) ?
+			    ((gfloat)scan_ctx->nscanned / (gfloat)scan_ctx->total) : 0.0f,
 			TRUE
 		    );
 
 		    g_free(path);
 		    g_free(pfx);
 		}
+		/* Scanned file? */
+		else if(g_strcasepfx(s, "Scanned file:"))
+		{
+		    /* Format is:
+		     *
+		     * "<pfx>: <nscanned> <blocks_scanned> <total_blocks_scanned>"
+		     */
+		    const gchar *s2 = (gchar *)strchr((char *)s, ':');
+
+		    /* Get the number of objects scanned */
+		    if(s2 != NULL)
+		    {
+			s2 = SEEK_PAST_ARG(s2);
+			s2 = SEEK_PAST_SPACES(s2);
+			scan_ctx->nscanned = (gulong)ATOL(s2);
+		    }
+
+		    /* Get the data scanned for this object in blocks */
+		    if(s2 != NULL)
+		    {
+			s2 = SEEK_PAST_ARG(s2);
+			s2 = SEEK_PAST_SPACES(s2);
+			/* Ignore this */
+		    }
+
+		    /* Get the total data scanned in blocks */
+		    if(s2 != NULL)
+		    {
+			s2 = SEEK_PAST_ARG(s2);
+			s2 = SEEK_PAST_SPACES(s2);
+			scan_ctx->blocks_scanned = (gulong)ATOL(s2);
+		    }
+
+		    WinUpdateTitle(win);
+
+		    /* Update the status bar progress */
+		    WinStatusProgress(
+			win,
+			(scan_ctx->total > 0l) ?
+			    ((gfloat)scan_ctx->nscanned / (gfloat)scan_ctx->total) : 0.0f,
+			TRUE
+		    );
+		}
 		/* Scanning directory? */
 		else if(g_strcasepfx(s, "Scanning directory:"))
 		{
@@ -1322,7 +1415,10 @@ gint WinScanMonitorTOCB(gpointer data)
 		     * "<pfx>: "<path>""
 		     */
 		    const gchar *s2 = s;
-		    gchar *pfx = NULL, *path = NULL, *msg;
+		    gchar	*pfx = NULL,
+				*path = NULL,
+				*shortened_path = NULL,
+				*msg;
 
 		    /* Get the prefix */
 		    if(s2 != NULL)
@@ -1333,23 +1429,32 @@ gint WinScanMonitorTOCB(gpointer data)
 		    /* Get the path */
 		    if(s2 != NULL)
 		    {
-			gchar *path2;
-
-			s2 = strchr(s2, '"');
+			/* Get the path */
+			s2 = (const gchar *)strchr((const char *)s2, '"');
 			if(s2 != NULL)
 			    s2++;
 			s2 = GET_ARG(s2, '"', &path);
-
-			/* Shorten the path as needed */
-			path2 = SHORTEN_STRING(path, 45);
-			g_free(path);
-			path = path2;
 		    }
 
+		    WinUpdateTitle(win);
+
+		    /* Update the results status */
+		    shortened_path = SHORTEN_STRING(path, 27);
 		    msg = g_strdup_printf(
 			"%s: %s",
-			pfx, path
+			pfx, shortened_path
 		    );
+		    g_free(shortened_path);
+		    WinResultsStatusUpdate(win, msg);
+		    g_free(msg);
+
+		    /* Update the status bar message */
+		    shortened_path = SHORTEN_STRING(path, 45);
+		    msg = g_strdup_printf(
+			"%s: %s",
+			pfx, shortened_path
+		    );
+		    g_free(shortened_path);
 		    WinStatusMessage(win, msg, TRUE);
 		    g_free(msg);
 
@@ -1357,87 +1462,127 @@ gint WinScanMonitorTOCB(gpointer data)
 		    g_free(pfx);
 #endif
 		}
-		/* Report */
-		else if(g_strcasepfx(s, "Report:"))
+		/* Clean */
+		else if(g_strcasepfx(s, "Clean:"))
 		{
-		    /* Seek past prefix */
-		    s = strchr(s, ':');
-		    s++;
-		    while(ISBLANK(*s))
-			s++;
+		    /* Format is:
+		     *
+		     * "<pfx>: "<path>""
+		     */
+		    gint new_row;
+		    const gchar *s2 = (gchar *)strchr((char *)s, ':');
+		    gchar	*path = NULL;
 
-		    /* Replace all ';' with newline characters */
-		    s2 = strchr(s, ';');
-		    while(s2 != NULL)
+		    /* Get the path */
+		    if(s2 != NULL)
 		    {
-			*s2 = '\n';
-			s2 = strchr(s, ';');
+			s2 = SEEK_PAST_ARG(s2);
+			s2 = SEEK_PAST_SPACES(s2);
+			s2 = (const gchar *)strchr((const char *)s2, '"');
+			if(s2 != NULL)
+			    s2++;
+			s2 = GET_ARG(s2, '"', &path);
 		    }
 
-#ifdef HAVE_EDV2
-		    /* Play "completed" sound */
-		    EDVPlaySoundCompleted(core->edv_ctx);
-#endif
-		    /* Raise Win */
-		    WinMap(win);
-
-		    WinStatusProgress(win, 1.0f, FALSE);
-		    WinStatusMessage(win, "Scan done", FALSE);
-
-		    /* Display message in dialog */
-		    if(!CDialogIsQuery())
+		    /* Add this to the results list */
+		    new_row = WinResultsListAppend(win, path, "Clean", FALSE);
+		    if(new_row > -1)
 		    {
-			CDialogSetTransientFor(toplevel);
-			CDialogGetResponseIconData(
-			    "Scan Results", s, NULL,
-			    (guint8 **)icon_avscan_32x32_xpm,
-			    CDIALOG_BTNFLAG_OK,
-			    CDIALOG_BTNFLAG_OK
-			);
-			CDialogSetTransientFor(NULL);
+			gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
 		    }
+
+		    WinUpdateTitle(win);
+
+		    g_free(path);
 		}
-		/* Report infected or errors */
-		else if(g_strcasepfx(s, "ReportInfected:") ||
-		        g_strcasepfx(s, "ReportError:")
-		)
+		/* Infected */
+		else if(g_strcasepfx(s, "Infected:"))
 		{
-		    /* Seek past prefix */
-		    s = strchr(s, ':');
-		    s++;
-		    while(ISBLANK(*s))
-			s++;
+		    /* Format is:
+		     *
+		     * "<pfx>: "<path>" <virus>"
+		     */
+		    gint new_row;
+		    const gchar *s2 = (gchar *)strchr((char *)s, ':');
+		    gchar	*path = NULL,
+				*virus = NULL;
 
-		    /* Replace all ';' with newline characters */
-		    s2 = strchr(s, ';');
-		    while(s2 != NULL)
+		    /* Get the path */
+		    if(s2 != NULL)
 		    {
-			*s2 = '\n';
-			s2 = strchr(s, ';');
+			s2 = SEEK_PAST_ARG(s2);
+			s2 = SEEK_PAST_SPACES(s2);
+			s2 = (const gchar *)strchr((const char *)s2, '"');
+			if(s2 != NULL)
+			    s2++;
+			s2 = GET_ARG(s2, '"', &path);
 		    }
 
-#ifdef HAVE_EDV2
-		    /* Play "warning" sound */
-		    EDVPlaySoundWarning(core->edv_ctx);
-#endif
-		    /* Raise Win */
-		    WinMap(win);
-
-		    WinStatusProgress(win, 1.0f, FALSE);
-		    WinStatusMessage(win, "Scan done", FALSE);
-
-		    /* Display message in dialog */
-		    if(!CDialogIsQuery())
+		    /* Get the virus name */
+		    if(s2 != NULL)
 		    {
-			CDialogSetTransientFor(toplevel);
-			CDialogGetResponseIconData(
-			    "Scan Results", s, NULL,
-			    (guint8 **)icon_biohazard_32x32_xpm,
-			    CDIALOG_BTNFLAG_OK,
-			    CDIALOG_BTNFLAG_OK
-			);
-			CDialogSetTransientFor(NULL);
+			s2 = SEEK_PAST_SPACES(s2);
+			virus = STRDUP(s2);
 		    }
+
+		    /* Count this as an infected file */
+		    scan_ctx->ninfected++;
+
+		    /* Add this to the results list */
+		    new_row = WinResultsListAppend(win, path, virus, TRUE);
+		    if(new_row > -1)
+		    {
+			gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
+		    }
+
+		    WinUpdateTitle(win);
+
+		    g_free(path);
+		    g_free(virus);
+		}
+		/* Problem */
+		else if(g_strcasepfx(s, "Problem:"))
+		{
+		    /* Format is:
+		     *
+		     * "<pfx>: "<path>" <problem>"
+		     */
+		    gint new_row;
+		    const gchar *s2 = (gchar *)strchr((char *)s, ':');
+		    gchar	*path = NULL,
+				*problem = NULL;
+
+		    /* Get the path */
+		    if(s2 != NULL)
+		    {
+			s2 = SEEK_PAST_ARG(s2);
+			s2 = SEEK_PAST_SPACES(s2);
+			s2 = strchr(s2, '"');
+			if(s2 != NULL)
+			    s2++;
+			s2 = GET_ARG(s2, '"', &path);
+		    }
+
+		    /* Get the problem */
+		    if(s2 != NULL)
+		    {
+			s2 = SEEK_PAST_SPACES(s2);
+			problem = STRDUP(s2);
+		    }
+
+		    scan_ctx->nproblems++;
+
+		    /* Add this to the results list */
+		    new_row = WinResultsListAppend(win, path, problem, FALSE);
+		    if(new_row > -1)
+		    {
+			gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
+		    }
+
+		    WinUpdateTitle(win);
+
+		    g_free(path);
+		    g_free(problem);
 		}
 		/* All else print to the status bar's message */
 		else
@@ -1451,34 +1596,101 @@ gint WinScanMonitorTOCB(gpointer data)
 	/* Process has exited? */
 	if(!WinScanProcessIsRunning(win))
 	{
+	    const gulong duration_t = scan_ctx->duration;
+	    gchar *duration_s;
+
 	    /* Stop scanning */
-	    win->scan_toid = 0;
+	    scan_ctx->toid = 0;
 	    WinScanProcessStop(win);
 
+	    /* Format the duration string
+	     *
+	     * Longer than 1 hour?
+	     */
+	    if(duration_t >= (60l * 60l))
+		duration_s = g_strdup_printf(
+		    "%i:%.2i:%.2i",
+		    (gint)(duration_t / (60l * 60l)),
+		    (gint)((duration_t / 60) % 60l),
+		    (gint)(duration_t % 60l)
+		);
+	    else	/* Less than 1 hour */
+		duration_s = g_strdup_printf(
+		    "%i:%.2i",
+		    (gint)(duration_t / 60l),
+		    (gint)(duration_t % 60l)
+		);
+
+	    /* Update the results status */
+	    if((scan_ctx->ninfected > 0l) || (scan_ctx->nproblems > 0l))
+	    {
+		gchar *msg = g_strdup_printf(
+		    "%ld %s Found   %ld %s Found   Duration: %s",
+		    scan_ctx->ninfected,
+		    (scan_ctx->ninfected == 1l) ? "Virus" : "Viruses",
+		    scan_ctx->nproblems,
+		    (scan_ctx->nproblems == 1l) ? "Problem" : "Problems",
+		    duration_s
+		);
+		WinResultsStatusUpdate(win, msg);
+		g_free(msg);
+	    }
+	    else
+	    {
+		gchar *msg = g_strdup_printf(
+		    "Clean,    Duration: %s",
+		    duration_s
+		);
+		WinResultsStatusUpdate(win, msg);
+		g_free(msg);
+	    }
+
+	    g_free(duration_s);
+
+	    if(scan_ctx->ninfected > 0l)
+	    {
+#ifdef HAVE_EDV2
+		/* Play the "completed" sound */
+		EDVPlaySoundCompleted(core->edv_ctx);
+#endif
+	    }
+	    else
+	    {
+#ifdef HAVE_EDV2
+		/* Play "warning" sound */
+		EDVPlaySoundWarning(core->edv_ctx);
+#endif
+	    }
+
+	    /* Update the status message */
 	    WinStatusMessage(
 		win,
 "Scan done",
 		FALSE
 	    );
 
+	    /* Raise the Win */
+	    WinMap(win);
+
+	    /* Display the report */
+	    WinResultsReportCB(win->toplevel, win);
+
 	    return(FALSE);
 	}
 
 	/* Update the progress only if the total number of files has
 	 * not been received yet
 	 */
-	if(win->scan_files_total == 0l)
+	if((scan_ctx->total == 0l) && !win->paused)
 	    WinStatusProgress(win, -1.0f, FALSE);
 
-	/* Update the time */
-	if(win->scan_start_time > 0l)
+	/* Update the duration */
+	if(scan_ctx->start_time > 0l)
 	{
 	    const gulong t = (gulong)time(NULL);
-	    WinStatusTime(
-		win,
-		(t > win->scan_start_time) ? (t - win->scan_start_time) : 0l,
-		FALSE
-	    );
+	    scan_ctx->duration = (t > scan_ctx->start_time) ?
+		(t - scan_ctx->start_time) : 0l;
+	    WinStatusTime(win, scan_ctx->duration, FALSE);
 	}
 
 	return(TRUE);
@@ -1490,7 +1702,7 @@ gint WinScanMonitorTOCB(gpointer data)
  *	Monitors the virus database update process and prints its
  *	stdout and stderr messages to the status bar.
  */
-gint WinUpdateMonitorTOCB(gpointer data)
+gint WinDBUpdateNetMonitorTOCB(gpointer data)
 {
 	gint p;
 	gchar buf[1024];
@@ -1501,7 +1713,7 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	if(win == NULL)
 	    return(FALSE);
 
-	p = win->update_pid;
+	p = win->db_update_net_pid;
 	toplevel = win->toplevel;
 	core = CORE(win->core);
 
@@ -1509,8 +1721,8 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	if(win->stop_count > 0)
 	{
 	    /* Stop update */
-	    win->update_toid = 0;
-	    WinUpdateProcessStop(win);
+	    win->db_update_net_toid = 0;
+	    WinDBUpdateNetProcessStop(win);
 
 	    WinStatusMessage(win, "Virus database update interrupted", FALSE); 
 	    return(FALSE);
@@ -1522,16 +1734,16 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	{
 	    gchar *s;
 
-	    while(fgets(buf, sizeof(buf), fp) != NULL)
+	    while(fgets((char *)buf, sizeof(buf), fp) != NULL)
 	    {
 		buf[sizeof(buf) - 1] = '\0';
 
 		/* Strip newline characters */
-		s = strpbrk(buf, "\n\r");
+		s = (gchar *)strpbrk((char *)buf, "\n\r");
 		if(s != NULL)
 		    *s = '\0';
 
-		/* Seek past spaces */
+		/* Seek past any spaces */
 		s = buf;
 		while(ISBLANK(*s))
 		    s++;
@@ -1544,39 +1756,88 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	fp = win->stderr_fp;
 	if(fp != NULL)
 	{
-	    gint new_row;
+	    gint nerrors = 0;
 	    gchar *s;
-	    GtkCList *clist = GTK_CLIST(win->results_clist);
 
-	    while(fgets(buf, sizeof(buf), fp) != NULL)
+	    while(fgets((char *)buf, sizeof(buf), fp) != NULL)
 	    {
 		buf[sizeof(buf) - 1] = '\0';
 
+		nerrors++;
+
 		/* Strip newline characters */
-		s = strpbrk(buf, "\n\r");
+		s = (gchar *)strpbrk((char *)buf, "\n\r");
 		if(s != NULL)
 		    *s = '\0';
 
-		/* Seek past spaces */
+		/* Seek past any spaces */
 		s = buf;
 		while(ISBLANK(*s))
 		    s++;
 
-		new_row = WinResultsListAppend(
-		    win,
-		    NULL,			/* No path */
-		    s				/* Virus/Problem */
+		/* Print the error message to the user */
+#ifdef HAVE_EDV2
+		EDVPlaySoundWarning(core->edv_ctx);
+#endif
+		CDialogSetTransientFor(toplevel);
+		CDialogGetResponse(
+		    "Update Virus Database Error",
+		    s,
+		    NULL,
+		    CDIALOG_ICON_WARNING,
+		    CDIALOG_BTNFLAG_OK,
+		    CDIALOG_BTNFLAG_OK
 		);
-		if(new_row > -1)
+		CDialogSetTransientFor(NULL);
+
+		/* If more than 10 errors have occured then query
+		 * user to stop
+		 */
+		if((nerrors % 10) == 0)
 		{
-		    gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
+		    gint response;
+		    gchar *msg = g_strdup_printf(
+"%i errors have occured during the virus database\n\
+update.\n\
+\n\
+Do you want to stop the virus database update?",
+			nerrors
+		    );
+#ifdef HAVE_EDV2
+                    EDVPlaySoundWarning(core->edv_ctx);
+#endif
+		    CDialogSetTransientFor(toplevel);
+		    response = CDialogGetResponse(
+                        "Update Virus Database Error",
+			msg,
+			NULL,
+                        CDIALOG_ICON_WARNING,
+                        CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
+                        CDIALOG_BTNFLAG_NO
+                    );
+		    g_free(msg);
+                    CDialogSetTransientFor(NULL);
+
+		    if(response == CDIALOG_RESPONSE_YES)
+		    {
+			/* Stop update */
+			win->db_update_net_toid = 0;
+			WinDBUpdateNetProcessStop(win);
+
+			WinStatusMessage(
+			    win,
+"Virus database update interrupted",
+			    FALSE
+			);
+			return(FALSE);
+		    }
 		}
 	    }
 	}
 
 
-	/* Process has exited? */ 
-	if(!WinUpdateProcessIsRunning(win))
+	/* Has the virus database update process exited? */ 
+	if(!WinDBUpdateNetProcessIsRunning(win))
 	{
 	    /* Mark the virus database update time */
 	    CFGItemListSetValueL(
@@ -1585,47 +1846,84 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	    );
 
 #ifdef HAVE_EDV2
-	    /* Play "completed" sound */
+	    /* Play the "completed" sound */
 	    EDVPlaySoundCompleted(core->edv_ctx);
 #endif
 
 	    /* Stop the virus database update */
-	    win->update_toid = 0;
-	    WinUpdateProcessStop(win);
+	    win->db_update_net_toid = 0;
+	    WinDBUpdateNetProcessStop(win);
 
 	    WinStatusMessage(win, "Virus database update done", TRUE);
 
-
-	    /* Clear Virus Database Tree due to virus database update */
-	    WinDBClear(win);
-
-	    /* Currently viewing the Virus Database Page? */
-	    if(win->page_num == WIN_PAGE_NUM_DB)
+	    /* Was there a change in the virus database or that the
+	     * virus database was not previously displayed or is
+	     * empty?
+	     */
+	    if(WinDBChanged(win) != 0)
 	    {
-		/* Need to update the Virus Database Tree since it is
-		 * currently being viewed
-		 *
-		 * Note that the Win will be raised after the update
-		 */
-		WinDBQueueUpdate(win);
+		/* Clear the Virus Database Tree due to virus database update */
+		WinDBClear(win);
+
+		/* Currently viewing the Virus Database Page? */
+		if(win->page_num == WIN_PAGE_NUM_DB)
+		{
+		    /* Need to update the Virus Database Tree since it
+		     * is currently being viewed
+		     *
+		     * Note that the Win will be raised after the update
+		     */
+		    WinDBQueueUpdate(win);
+		}
 	    }
 
 	    return(FALSE);
 	}
 
 	/* Update the progress */
-	WinStatusProgress(win, -1.0f, FALSE);
+	if(!win->paused)
+	    WinStatusProgress(win, -1.0f, FALSE);
 
 	/* Update the time */
-	if(win->update_start_time > 0l)
+	if(win->db_update_net_start_time > 0l)
 	{
 	    const gulong t = (gulong)time(NULL);
 	    WinStatusTime(
 		win,
-		(t > win->update_start_time) ? (t - win->update_start_time) : 0l,
+		(t > win->db_update_net_start_time) ?
+		    (t - win->db_update_net_start_time) : 0l,
 		FALSE
 	    );
 	}
 
 	return(TRUE);
 }
+
+
+/*
+ *	Pattern dialog "delete_event" signal callback.
+ */
+gint WinPatternDlgDeleteEventCB(
+	GtkWidget *widget, GdkEvent *event, gpointer data
+)
+{
+	win_pattern_dlg_struct *d = WIN_PATTERN_DLG(data);
+	if(d == NULL)
+	    return(FALSE);
+
+	gtk_widget_hide(d->toplevel);
+
+	return(TRUE);
+}
+
+/*
+ *	Pattern dialog close signal callback.
+ */
+void WinPatternDlgCloseCB(GtkWidget *widget, gpointer data)
+{
+	win_pattern_dlg_struct *d = WIN_PATTERN_DLG(data);
+	if(d == NULL)
+	    return;
+
+	gtk_widget_hide(d->toplevel);
+}
