@@ -22,7 +22,9 @@ extern void WinScanListUpdateRow(
 
 /* Results List */
 extern gint WinResultsListAppend(
-	win_struct *win, const gchar *path, const gchar *virus_name
+	win_struct *win,
+	const gchar *path, const gchar *virus_name,
+	const gboolean infected
 );
 extern void WinResultsListClear(win_struct *win);
 
