@@ -20,21 +20,25 @@ extern void WinInvertSelectionCB(GtkWidget *widget, gpointer data);
 
 extern void WinStartCB(GtkWidget *widget, gpointer data);
 extern void WinStopCB(GtkWidget *widget, gpointer data);
+extern void WinPauseResumeCB(GtkWidget *widget, gpointer data);
 extern void WinLocationCB(GtkWidget *widget, gpointer data);
 extern void WinRecursiveCB(GtkWidget *widget, gpointer data);
 extern void WinExecutablesOnlyCB(GtkWidget *widget, gpointer data);
 extern void WinIgnoreLinksCB(GtkWidget *widget, gpointer data);
+extern void WinScannerSettingsCB(GtkWidget *widget, gpointer data);
 
 extern void WinResultsRecycleCB(GtkWidget *widget, gpointer data);
 extern void WinResultsMoveCB(GtkWidget *widget, gpointer data);
 extern void WinResultsSelectAllCB(GtkWidget *widget, gpointer data);
 extern void WinResultsUnselectAllCB(GtkWidget *widget, gpointer data);
 extern void WinResultsInvertSelectionCB(GtkWidget *widget, gpointer data);
+extern void WinResultsReportCB(GtkWidget *widget, gpointer data);
 extern void WinResultsOpenCB(GtkWidget *widget, gpointer data);
 extern void WinResultsSaveAsCB(GtkWidget *widget, gpointer data);
 extern void WinResultsClearCB(GtkWidget *widget, gpointer data);
 
 extern void WinDBRefreshCB(GtkWidget *widget, gpointer data);
+extern void WinDBPatternDetailsCB(GtkWidget *widget, gpointer data);
 extern void WinDBLocationCB(GtkWidget *widget, gpointer data);
 extern void WinDBUpdateNetCB(GtkWidget *widget, gpointer data);
 
